SystemOrganization addCategory: #'Zinc-Character-Encoding-Core'!

!String methodsFor: '*Zinc-Character-Encoding-Core' stamp: 'SvenVanCaekenberghe 9/15/2014 13:35'!
asZnCharacterEncoder
	"Return a ZnCharacterEncoder instance using the receiver as identifier"
	
	" 'UTF-8' asZnCharacterEncoder "
	
	^ ZnCharacterEncoder newForEncoding: self! !

!String methodsFor: '*Zinc-Character-Encoding-Core' stamp: 'SvenVanCaekenberghe 9/15/2014 13:35'!
encodeWith: encoding
	"Produce a ByteArray that encodes the receiver, using a specified encoding.
	Encoding is either a ZnCharacterEncoder instance or an identifier for one."
	
	" 'Les élèves français' encodeWith: #utf8 "
	
	^ encoding asZnCharacterEncoder encodeString: self! !

!String methodsFor: '*Zinc-Character-Encoding-Core' stamp: 'SvenVanCaekenberghe 9/15/2014 11:48'!
urlDecoded
	"URL Decode the receiver and return the resulting String.
	This is an encoding where characters that are illegal in a URL are escaped."

    ^ ZnPercentEncoder new decode: self! !

!String methodsFor: '*Zinc-Character-Encoding-Core' stamp: 'SvenVanCaekenberghe 9/15/2014 11:48'!
urlEncoded
	"URL Encode the receiver and return the resulting String.
	This is an encoding where characters that are illegal in a URL are escaped."

    ^ ZnPercentEncoder new encode: self
! !

!String methodsFor: '*Zinc-Character-Encoding-Core' stamp: 'SvenVanCaekenberghe 9/15/2014 13:36'!
utf8Encoded
	"Produce a ByteArray encoding the receiver using UTF-8,
	the recommended encoding for Strings, unless you know what you are doing."
	
	" 'Les élèves français' utf8Encoded "
	
	^ self encodeWith: #utf8! !

!ByteArray methodsFor: '*Zinc-Character-Encoding-Core' stamp: 'SvenVanCaekenberghe 9/15/2014 13:37'!
decodeWith: encoding
	"Produce a String that decodes the receiver, using a specified encoding.
	Encoding is either a ZnCharacterEncoder instance or an identifier for one."
	
	"#[76 101 115 32 195 169 108 195 168 118 101 115 32 102 114 97 110 195 167 97 105 115] decodeWith: #utf8"
	
	^ encoding asZnCharacterEncoder decodeBytes: self! !

!ByteArray methodsFor: '*Zinc-Character-Encoding-Core' stamp: 'SvenVanCaekenberghe 9/15/2014 13:37'!
utf8Decoded
	"Produce a String decoding the receiver using UTF-8,
	the recommended encoding for Strings, unless you know what you are doing."

	"#[76 101 115 32 195 169 108 195 168 118 101 115 32 102 114 97 110 195 167 97 105 115] utf8Decoded"
	
	^ self decodeWith: #utf8! !

DynamicVariable subclass: #ZnDefaultCharacterEncoder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-Character-Encoding-Core'!

!ZnDefaultCharacterEncoder commentStamp: 'SvenVanCaekenberghe 5/10/2017 17:10' prior: 0!
I am ZnDefaultCharacterEncoder.
I am a DynamicVariable and a ProcessSpecificVariable.

I can be used to modify the default ZnCharacteEncoder on a per process basis, for example:

ZnDefaultCharacterEncoder 
	value: ZnUTF8Encoder new
	during: [ ZnClient new get: 'http://zn.stfx.eu/zn/small.html' ]!

!ZnDefaultCharacterEncoder methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/10/2017 17:09'!
default
	^ ZnUTF8Encoder new! !

Error subclass: #ZnByteStringBecameWideString
	instanceVariableNames: 'byteString wideString'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-Character-Encoding-Core'!

!ZnByteStringBecameWideString commentStamp: '<historical>' prior: 0!
I am ZnByteStringBecameWideString, a resumable Error signalled to indicate that some byteString was changed to a wideString.

Used by ZnUTF8Encoder>>#readInto:startingAt:count:fromStream: to avoid a #becomeForward: when a ByteString automagically changes into a WideString.

Part of Zinc HTTP Components.!

!ZnByteStringBecameWideString class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 11/13/2013 13:49'!
convert: byteString
	"Convert byteString to a wideString, signalling a resumable error"
	
	| wideString |
	wideString := WideString from: byteString.
	self new
		byteString: byteString;
		wideString: wideString;
		signal.
	^ wideString! !

!ZnByteStringBecameWideString methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 11/13/2013 14:28'!
becomeForward
	"Switch the identities of byteString and wideString using #becomeForward:"

	byteString becomeForward: wideString ! !

!ZnByteStringBecameWideString methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/11/2013 14:44'!
byteString
	^ byteString! !

!ZnByteStringBecameWideString methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/11/2013 14:44'!
byteString: anObject
	byteString := anObject! !

!ZnByteStringBecameWideString methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 11/13/2013 13:26'!
isResumable
	^ true! !

!ZnByteStringBecameWideString methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/11/2013 14:44'!
wideString
	^ wideString! !

!ZnByteStringBecameWideString methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/11/2013 14:44'!
wideString: anObject
	wideString := anObject! !

Error subclass: #ZnCharacterEncodingError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-Character-Encoding-Core'!

!ZnCharacterEncodingError commentStamp: '<historical>' prior: 0!
I am ZnCharacterEncodingError.
I am an Error.

I signal when something goes wrong while encoding or decoding characters.

Part of Zinc HTTP Components!

ZnCharacterEncodingError subclass: #ZnInvalidUTF8
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-Character-Encoding-Core'!

!ZnInvalidUTF8 commentStamp: '<historical>' prior: 0!
I am ZnInvalidUTF8.
I am a ZnCharacterEncodingError.
I am an Error.

I signal when something goes wrong while encoding or decoding UTF8.

Part of Zinc HTTP Components!

Object subclass: #ZnBase64Encoder
	instanceVariableNames: 'alphabet inverse lineLength lineEnd whitespace'
	classVariableNames: 'DefaultAlphabet DefaultInverse'
	poolDictionaries: ''
	category: 'Zinc-Character-Encoding-Core'!

!ZnBase64Encoder commentStamp: 'SvenVanCaekenberghe 10/30/2015 15:06' prior: 0!
I am ZnBase64Encoder.

Base64 encoding is a technique to encode binary data as a string of characters that can be safely transported over various protocols. Basically, every 3 bytes are encoded using 4 characters from an alphabet of 64. Each encoded character represents 6 bits.

The most commonly used alphabet is 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/'. One or two equal signs (= or ==) are used for padding.

  ZnBase64Encoder new encode: #[0 1 2 3 4 5].
  ZnBase64Encoder new encode: #[10 20]
  ZnBase64Encoder new decode: 'BQQDAgEA'.
  ZnBase64Encoder new decode: 'FAo='.

The encoded data can optionally be broken into lines. Characters not part of the alphabet are considered as white space and are ignored when inbetween groups of 4 characters.

My #encode: method works from ByteArray to String, while my #decode: method works from String to ByteArray.

Note that to encode a String as Base64, you first have to encode the characters as bytes using a character encoder.

See also http://en.wikipedia.org/wiki/Base64

Part of Zinc HTTP Components.!

!ZnBase64Encoder class methodsFor: 'class initialization' stamp: 'SvenVanCaekenberghe 12/16/2012 12:54'!
initialize
	DefaultAlphabet := String withAll: ($A to: $Z) , ($a to: $z) , ($0 to: $9) , #($+ $/).
	DefaultInverse := Array new: 128.
	0 to: 127 do: [ :each | 
		| offset |
		offset := DefaultAlphabet indexOf: each asCharacter ifAbsent: [ nil ].
		DefaultInverse at: each + 1 put: (offset ifNotNil: [ offset - 1 ]) ]! !

ZnBase64Encoder initialize!

!ZnBase64Encoder methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 10/30/2015 14:58'!
alphabet
	"Return the alphabet that I am using to encode byte values"
	
	^ alphabet ! !

!ZnBase64Encoder methodsFor: 'initialization' stamp: 'SvenVanCaekenberghe 12/16/2012 15:15'!
alphabet: string
	"Set the alphabet to use to string, containing 64 characters to represent 64 byte values.
	I automatically compute the inverse used for fast decoding."
	
	self assert: string size = 64 description: '64 characters are needed for a Base64 alphabet'.
	alphabet := string.
	inverse := Array new: 128.
	0 to: 127 do: [ :each | 
		| offset |
		offset := alphabet indexOf: each asCharacter ifAbsent: [ nil ].
		inverse at: each + 1 put: (offset ifNotNil: [ offset - 1 ]) ]! !

!ZnBase64Encoder methodsFor: 'initialization' stamp: 'SvenVanCaekenberghe 12/15/2012 14:06'!
breakLines
	"Configure me to break lines and insert newlines every 76 characters while encoding"

	self breakLinesAt: 76! !

!ZnBase64Encoder methodsFor: 'initialization' stamp: 'SvenVanCaekenberghe 12/16/2012 15:13'!
breakLinesAt: length
	"Configure me to break lines at lenth, a multiple of 4, and insert newlines"

	self assert: (length \\ 4) isZero description: 'line length should be a multiple of 4'.
	lineLength := length.
	lineEnd ifNil: [ self lineEndConvention: #crlf ]! !

!ZnBase64Encoder methodsFor: 'private' stamp: 'SvenVanCaekenberghe 10/30/2015 13:25'!
byteCountFor: string	
	| n byteCount |
	"This assumes there are no line breaks in string"
	n := string size.
	byteCount := n // 4 * 3.
	(n > 1 and: [ (string at: n) = $= ])
		ifTrue: [ 
			(n > 2 and: [ (string at: n - 1) = $= ])
				ifTrue: [ byteCount := byteCount - 2 ]
				ifFalse: [ byteCount := byteCount - 1 ] ].
	^ byteCount! !

!ZnBase64Encoder methodsFor: 'private' stamp: 'SvenVanCaekenberghe 10/30/2015 13:25'!
characterCountFor: bytes
	| n characterCount | 
	n := bytes size.
	characterCount := n // 3 + (n \\ 3) sign * 4.
	^ lineLength isNil
		ifTrue: [ characterCount ]
		ifFalse: [ characterCount + ((characterCount // lineLength ) * lineEnd size) ]! !

!ZnBase64Encoder methodsFor: 'private' stamp: 'SvenVanCaekenberghe 10/30/2015 13:30'!
characterForValue: value
	^ alphabet at: value + 1! !

!ZnBase64Encoder methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 10/30/2015 13:18'!
decode: string
	"Decode a Base64 encoded string and return the resulting byte array"
	
	^ ByteArray 
		new: (self byteCountFor: string)
		streamContents: [ :byteStream | 
			self decode: string readStream to: byteStream ]! !

!ZnBase64Encoder methodsFor: 'private' stamp: 'SvenVanCaekenberghe 10/30/2015 16:10'!
decode: char1 and: char2 and: char3 and: char4 to: stream
	| v1 v2 v3 v4 |
	v1 := self valueForCharacter: char1.
	v2 := self valueForCharacter: char2.
	stream nextPut: (v1 bitShift: 2) + (v2 bitShift: -4).
	char3 == $=
		ifFalse: [ 
			v3 := self valueForCharacter: char3.
			stream nextPut: ((v2 bitAnd: 2r1111) bitShift: 4) + (v3 bitShift: -2).
			char4 == $=
				ifFalse: [ 
						v4 := self valueForCharacter: char4.
						stream nextPut: ((v3 bitAnd: 2r11) bitShift: 6) + v4 ] ]! !

!ZnBase64Encoder methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 4/27/2017 10:12'!
decode: stringStream to: byteStream
	| char1 char2 char3 char4 |
	[ stringStream atEnd ] whileFalse: [ 
		self skipWhitespace: stringStream.
		stringStream atEnd ifTrue: [ ^ self ].
		char1 := stringStream next.
		char2 := stringStream next.
		char3 := stringStream next.
		char4 := stringStream next.
		char1 isNil | char2 isNil | char3 isNil | char4 isNil
			ifTrue: [ ZnCharacterEncodingError signal: 'Illegal Base64 input' ].
		self decode: char1 and: char2 and: char3 and: char4 to: byteStream ]! !

!ZnBase64Encoder methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 10/30/2015 13:18'!
encode: byteArray
	"Encode byteArray using Base64 encoding and return the resulting string"
	
	^ String 
		new: (self characterCountFor: byteArray) 
		streamContents: [ :stringStream | 
			self encode: byteArray readStream to: stringStream ]! !

!ZnBase64Encoder methodsFor: 'private' stamp: 'SvenVanCaekenberghe 10/30/2015 16:25'!
encode: byte1 and: byte2 and: byte3 to: stream
	stream nextPut: (self characterForValue: (byte1 bitShift: -2)).
	byte2
		ifNil: [ 
			stream nextPut: (self characterForValue: ((byte1 bitAnd: 2r11) bitShift: 4)).
			stream nextPutAll: '==' ]
		ifNotNil: [ 
			stream nextPut: (self characterForValue: (((byte1 bitAnd: 2r11) bitShift: 4) + (byte2 bitShift: -4))).
			byte3
				ifNil: [ 
					stream nextPut: (self characterForValue: ((byte2 bitAnd: 2r1111) bitShift: 2)).
					stream nextPut: $= ]
				ifNotNil: [ 
					stream nextPut: (self characterForValue: (((byte2 bitAnd: 2r1111) bitShift: 2) + (byte3 bitShift: -6))).
					stream nextPut: (self characterForValue: (byte3 bitAnd: 2r111111)) ] ]! !

!ZnBase64Encoder methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 10/30/2015 14:42'!
encode: byteStream to: stringStream
	| byte1 byte2 byte3 count |
	lineLength
		ifNil: [ 
			[ byteStream atEnd ] whileFalse: [ 
				byte1 := byteStream next.
				byte2 := byteStream next.
				byte3 := byteStream next.
				self encode: byte1 and: byte2 and: byte3 to: stringStream ] ]
		ifNotNil: [ 
			count := 0.
			[ byteStream atEnd ] whileFalse: [ 
				byte1 := byteStream next.
				byte2 := byteStream next.
				byte3 := byteStream next.
				self encode: byte1 and: byte2 and: byte3 to: stringStream.
				(count := count + 4) = lineLength
					ifTrue: [ 
						stringStream nextPutAll: lineEnd.
						count := 0 ] ] ]! !

!ZnBase64Encoder methodsFor: 'initialization' stamp: 'SvenVanCaekenberghe 4/27/2017 10:13'!
initialize
	super initialize.
	alphabet := DefaultAlphabet.
	inverse := DefaultInverse.
	self whitespace: #any! !

!ZnBase64Encoder methodsFor: 'private' stamp: 'SvenVanCaekenberghe 10/30/2015 16:52'!
isLegalCharacter: character
	"Return true when character is part of my alphabet"

	| code |
	^ (code := character asciiValue) < 128 
			and: [ (inverse at: code + 1) notNil ]! !

!ZnBase64Encoder methodsFor: 'private' stamp: 'SvenVanCaekenberghe 4/27/2017 10:21'!
isWhitespaceCharacter: character
	"Return true when character should be considered whitespace"

	whitespace 
		ifNil: [ "No whitespace allowed" 
			^ false ].
	whitespace = #separator
		ifTrue: [ "Only separators are considered whitespace" 
			^ character isSeparator ].
	whitespace = #any
		ifTrue: [ "All non-legal (non-alphabet) characters are considered whitespace" 
			^ (self isLegalCharacter: character) not ].
	^ false! !

!ZnBase64Encoder methodsFor: 'initialization' stamp: 'SvenVanCaekenberghe 12/15/2012 00:50'!
lineEndConvention: symbol
	"Set the end of line convention to be used.
	Either #cr, #lf or #crlf (the default)."
	
	self assert: (#(cr lf crlf) includes: symbol).
	lineEnd := String perform: symbol! !

!ZnBase64Encoder methodsFor: 'private' stamp: 'SvenVanCaekenberghe 4/27/2017 10:22'!
skipWhitespace: stream
	[ stream atEnd not and: [ (self isWhitespaceCharacter: stream peek) ] ] 
		whileTrue: [ stream next ]! !

!ZnBase64Encoder methodsFor: 'private' stamp: 'SvenVanCaekenberghe 10/30/2015 16:59'!
valueForCharacter: char
	| code |
	(code := char asciiValue) < 128
		ifTrue: [ 
			(inverse at: code + 1)
				ifNotNil: [ :byteValue | ^ byteValue ] ].
	ZnCharacterEncodingError signal: 'Illegal Base64 input'! !

!ZnBase64Encoder methodsFor: 'initialization' stamp: 'SvenVanCaekenberghe 4/27/2017 10:17'!
whitespace: mode
	"Set the whitespace mode: 
	nil is no whitespace allowed, 
	#separator is CR, LF, FF, SPACE, TAB allowed,
	#any is all non-alphabet characters allowed (the default)"
	
	self assert: (#(nil separator any) includes: mode).
	whitespace := mode! !

Object subclass: #ZnBufferedReadStream
	instanceVariableNames: 'stream buffer position limit'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-Character-Encoding-Core'!

!ZnBufferedReadStream commentStamp: '<historical>' prior: 0!
I am ZnBufferedReadStream.

I wrap another ReadStream and add efficient buffering for the typical access pattern of parsers: sending lots of #next, #peek and #atEnd messages.

By design I do not implement #position and #position: or anything based on that.

I can wrap both binary or character streams and act accordingly.

Part of Zinc HTTP Components.!

!ZnBufferedReadStream class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 11/30/2012 10:57'!
on: readStream
	^ self new
		on: readStream;
		yourself! !

!ZnBufferedReadStream class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 12/2/2012 19:23'!
on: readStream do: block
	"Execute block with as argument a ZnBufferedReadStream on readStream.
	Return the value of block."

	^ block value: (self on: readStream)! !

!ZnBufferedReadStream methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 11/30/2012 10:39'!
atEnd
	^ position > limit and: [ stream atEnd ]
	! !

!ZnBufferedReadStream methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 11/30/2012 10:28'!
close
	stream close! !

!ZnBufferedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 11/30/2012 20:18'!
collectionSpecies
	^ stream isBinary
		ifTrue: [ ByteArray ]
		ifFalse: [ String ]! !

!ZnBufferedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 11/30/2012 20:15'!
defaultBufferSize
	^ 2 raisedToInteger: 16! !

!ZnBufferedReadStream methodsFor: 'initialization' stamp: 'SvenVanCaekenberghe 11/30/2012 20:15'!
initialize
	super initialize.
	position := 1.
	limit := 0! !

!ZnBufferedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 11/30/2012 21:28'!
isBinary
	^ stream isBinary! !

!ZnBufferedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/1/2012 20:48'!
next
	"Return the next element and move over it"
	
	position > limit
		ifTrue: [ self nextBuffer ].
	^ position <= limit
		ifTrue: [ 
			| char |
			char := buffer at: position.
			position := position + 1.
			char ]
		ifFalse: [ nil ]! !

!ZnBufferedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 11/30/2012 20:18'!
next: requestedCount 
	"Read requestedCount elements and return them as a collection.
	If less are available, a smaller collection will be returned."

	^ self 
		next: requestedCount 
		into: (self collectionSpecies new: requestedCount)! !

!ZnBufferedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 11/30/2012 20:11'!
next: requestedCount into: collection
	"Read requestedCount elements into collection,
	returning a copy if less elements are available"
	
	^ self 
		next: requestedCount 
		into: collection 
		startingAt: 1   ! !

!ZnBufferedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 11/30/2012 20:11'!
next: requestedCount into: collection startingAt: offset
	"Read requestedCount elements into collection starting at offset,
	returning a copy if less elements are available"
	
	| read |
	read := self 
		readInto: collection 
		startingAt: offset 
		count: requestedCount.
	^ read = requestedCount 
		ifTrue: [ collection ]
		ifFalse: [ collection copyFrom: 1 to: offset + read - 1 ]     ! !

!ZnBufferedReadStream methodsFor: 'private' stamp: 'SvenVanCaekenberghe 11/30/2012 11:15'!
nextBuffer
	limit := stream readInto: buffer startingAt: 1 count: buffer size.
	position := 1! !

!ZnBufferedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 11/30/2012 20:11'!
nextInto: collection
	"Read the next elements of the receiver into collection,
	returning a copy if less elements are available"
	
	^ self
		next: collection size
		into: collection! !

!ZnBufferedReadStream methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 11/30/2012 20:15'!
on: readStream
	stream := readStream.
	self sizeBuffer: self defaultBufferSize! !

!ZnBufferedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/1/2012 20:48'!
peek
	"Return the next element but do not move over it"
	
	position > limit
		ifTrue: [ self nextBuffer ].
	^ position <= limit
		ifTrue: [ buffer at: position ]
		ifFalse: [ nil ]! !

!ZnBufferedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/1/2012 20:47'!
peekFor: object
	"Answer false and do not move over the next element if it is not equal to object, or if the receiver is at the end. 
	Answer true and move over the next element when it is equal to object."

	^ self peek = object
		ifTrue: [ 
			self next.
			true ]
		ifFalse: [ false ]! !

!ZnBufferedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 11/30/2012 22:16'!
readInto: collection startingAt: offset count: requestedCount
	"Read requestedCount elements into collection starting at offset,
	returning the number of elements read, there could be less elements available."

	| read |
	read := 0.
	position <= limit
		ifTrue: [ 
			read := (limit - position + 1) min: requestedCount.
			collection replaceFrom: offset to: offset + read - 1 with: buffer startingAt: position.
			position := position + read.
			read = requestedCount ifTrue: [ ^ requestedCount ] ].
	^ read
		+
			(requestedCount - read < (buffer size / 2)
				ifTrue: [ 
					stream atEnd
						ifTrue: [ 0 ]
						ifFalse: [ 
							self
								nextBuffer;
								readInto: collection startingAt: offset + read count: requestedCount - read ] ]
				ifFalse: [ stream readInto: collection startingAt: offset + read count: requestedCount - read ])! !

!ZnBufferedReadStream methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 11/30/2012 20:19'!
sizeBuffer: size
	buffer := self collectionSpecies new: size! !

!ZnBufferedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 11/30/2012 21:36'!
skip: count
	"Skip over count elements.
	This could be further optimzed."
	
	count timesRepeat: [ self next ]! !

!ZnBufferedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 11/30/2012 21:33'!
upTo: value 
	"Read upto but not including value and return them as a collection.
	If value is not found, return the entire contents of the stream.
	This could be further optimzed."
	
	^ self collectionSpecies 
		streamContents: [ :writeStream | | element |
			[ self atEnd or: [ (element := self next) = value ] ] whileFalse: [ 
				writeStream nextPut: element ] ]! !

!ZnBufferedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/16/2012 16:31'!
upToEnd
	"Read elements until the stream is atEnd and return them as a collection."

	^ self collectionSpecies
		streamContents: [ :out | 
			[ self atEnd ] whileFalse: [ 
				position > limit
					ifTrue: [ self nextBuffer ].	
				out next: limit - position + 1 putAll: buffer startingAt: position.
				position := limit + 1 ] ]! !

!ZnBufferedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 11/30/2012 10:27'!
wrappedStream
	^ stream! !

Object subclass: #ZnBufferedWriteStream
	instanceVariableNames: 'stream buffer position'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-Character-Encoding-Core'!

!ZnBufferedWriteStream commentStamp: '<historical>' prior: 0!
I am ZnBufferedWriteStream.
I wrap a write stream and add buffering.

Make sure to always send me #flush or #close when you're done,
otherwise the last buffer might not yet have been written.
My class side's #on:do: helps to ensure this.

I can wrap both binary or character streams and act accordingly.

Part of Zinc HTTP Components.!

!ZnBufferedWriteStream class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 12/7/2010 21:20'!
on: writeStream
	^ self basicNew
		on: writeStream;
		yourself! !

!ZnBufferedWriteStream class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 12/2/2012 19:23'!
on: writeStream do: block
	"Execute block with as argument a ZnBufferedWriteStream on writeStream,
	making sure #flush is called at the end. Return the value of block."
	
	| bufferedWriteStream result |
	bufferedWriteStream := self on: writeStream.
	result := block value: bufferedWriteStream.
	bufferedWriteStream flush.
	^ result! !

!ZnBufferedWriteStream methodsFor: 'private' stamp: 'SvenVanCaekenberghe 12/8/2010 10:15'!
buffer
	buffer isNil 
		ifTrue: [ self sizeBuffer: self defaultBufferSize ].
	^ buffer! !

!ZnBufferedWriteStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/7/2010 23:42'!
bufferFreeSize
	^ self bufferSize - position! !

!ZnBufferedWriteStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/8/2010 10:08'!
bufferSize
	^ buffer isNil 
		ifTrue: [ self defaultBufferSize ]
		ifFalse: [ buffer size ]! !

!ZnBufferedWriteStream methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 12/7/2010 21:22'!
close
	self flushBuffer.
	stream close! !

!ZnBufferedWriteStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/16/2017 10:34'!
cr
	self nextPut: Character cr! !

!ZnBufferedWriteStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/16/2017 10:34'!
crlf
	self cr; lf! !

!ZnBufferedWriteStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 11/30/2012 21:22'!
defaultBufferSize
	^ 2 raisedToInteger: 16! !

!ZnBufferedWriteStream methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 5/18/2013 20:17'!
finish
	self flushBuffer! !

!ZnBufferedWriteStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/7/2010 21:23'!
flush
	self flushBuffer.
	stream flush! !

!ZnBufferedWriteStream methodsFor: 'private' stamp: 'SvenVanCaekenberghe 12/7/2010 23:48'!
flushBuffer
	position = 0 ifTrue: [ ^ self ].
	position = self bufferSize
		ifTrue: [
			stream nextPutAll: buffer ]
		ifFalse: [
			(stream respondsTo: #next:putAll:startingAt:)
				ifTrue: [ stream next: position putAll: buffer startingAt: 1 ]
				ifFalse: [ stream nextPutAll: (buffer copyFrom: 1 to: position) ] ].
	position := 0! !

!ZnBufferedWriteStream methodsFor: 'private' stamp: 'SvenVanCaekenberghe 12/7/2010 23:39'!
flushBufferIfFull
	position = self bufferSize 
		ifTrue: [ self flushBuffer ]
! !

!ZnBufferedWriteStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/16/2017 10:34'!
lf
	self nextPut: Character lf! !

!ZnBufferedWriteStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 11/30/2012 20:01'!
next: count putAll: collection
	"Write count elements from collection"
	
	self 
		next: count 
		putAll: collection 
		startingAt: 1! !

!ZnBufferedWriteStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 11/30/2012 21:20'!
next: count putAll: collection startingAt: offset
	"Write count elements from collection starting at offset."
	
	self flushBufferIfFull.
	count <= self bufferFreeSize
		ifTrue: [
			self buffer replaceFrom: position + 1 to: position + count with: collection startingAt: offset.
			position := position + count ]
		ifFalse: [
			self flushBuffer.
			count > (self bufferSize / 2)
				ifTrue: [ stream next: count putAll: collection startingAt: offset ]
				ifFalse: [ self next: count putAll: collection startingAt: offset ] ]
! !

!ZnBufferedWriteStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/8/2010 10:16'!
nextPut: object
	self flushBufferIfFull.
	position := position + 1.
	self buffer at: position put: object! !

!ZnBufferedWriteStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 11/30/2012 20:02'!
nextPutAll: collection
	"Write a collection"
	
	self 
		next: collection size 
		putAll: collection 
		startingAt: 1! !

!ZnBufferedWriteStream methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 12/7/2010 21:30'!
on: writeStream
	stream := writeStream.
	position := 0! !

!ZnBufferedWriteStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 11/30/2012 22:21'!
print: object
	object printOn: self! !

!ZnBufferedWriteStream methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 12/7/2010 21:19'!
printOn: aStream
	aStream 
		nextPutAll: 'a '; 
		nextPutAll: self class name! !

!ZnBufferedWriteStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/7/2010 21:28'!
sizeBuffer: size
	buffer := (stream isBinary ifTrue: [ ByteArray ] ifFalse: [ String ]) new: size! !

!ZnBufferedWriteStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/7/2010 21:20'!
space
	self nextPut: Character space! !

!ZnBufferedWriteStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 10/3/2014 10:04'!
tab
	self nextPut: Character tab! !

Object subclass: #ZnCharacterEncoder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-Character-Encoding-Core'!

!ZnCharacterEncoder commentStamp: 'SvenVanCaekenberghe 1/15/2016 16:02' prior: 0!
I am ZnCharacterEncoder, I encode and decode Character objects to and from a binary stream.
I am an abstract class with following protocol:

#nextFromStream:
#nextPut:toStream:
#encodedByteCountFor:
#backOnStream:

The first two are compatible with TextConverter and subclasses.

I add some convenience methods:

#encodeString:
#decodeBytes:
#encodedByteCountForString:

Contrary to older encoders, I work strictly from strings to bytes and vice versa and I will throw errors instead of silently ignoring them.

I also implement optimized bulk operations:

#next:putAll:startingAt:toStream:
#readInto:startingAt:count:fromStream:

Additionally, I can encode Integer code points to a binary stream as well as read Integer code points from a binary stream. This is in a sense a more fundamental operation that avoids instanciating Character objects.

#nextCodePointFromStream:
#nextPutCodePoint:toStream:
#encodedByteCountForCodePoint:

#decodeAsCodePoints:
#encodeCodePoints:
#encodedByteCountForCodePoints:

Part of Zinc HTTP Components.!

!ZnCharacterEncoder class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 7/24/2015 14:58'!
ascii
	^ self newForEncoding: 'ASCII'! !

!ZnCharacterEncoder class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/9/2015 16:22'!
canonicalEncodingIdentifier: string
	^ (string select: [ :each | each isAlphaNumeric ]) asLowercase! !

!ZnCharacterEncoder class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/10/2017 17:11'!
default
	"Return the default ZnCharacterEncoder to be used 
	when none is otherwise specified."
	
	^ ZnDefaultCharacterEncoder value! !

!ZnCharacterEncoder class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 5/3/2017 15:39'!
detectEncoding: bytes
	"Return one of my instances capable of decoding bytes.
	This is done by successively trying known encodings in a specific order.
	If no one is found, signal ZnCharacterEncodingError.
	This is a heuristic and unreliable [https://en.wikipedia.org/wiki/Charset_detection]."
	
	| candidates |
	"Set up an ordered candidates list, 7-bit ascii and utf8 are reasonably reliable, iso88591 is a reasonable default"
	candidates := #(ascii utf8 iso88591).
	candidates := candidates , (ZnByteEncoder knownEncodingIdentifiers difference: candidates).
	candidates := candidates , (self knownEncodingIdentifiers difference: candidates).
	"Try each and return the first one that succeeeds."
	candidates do: [ :identifier | | encoder |
		encoder := self newForEncoding: identifier.
		[ ^ encoder decodeBytes: bytes; yourself ] on: ZnCharacterEncodingError do: [ ] ].
	ZnCharacterEncodingError signal: 'No suitable encoder found'! !

!ZnCharacterEncoder class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/25/2011 11:07'!
handlesEncoding: string
	"Return true when my instances handle the encoding described by string"
	
	self subclassResponsibility ! !

!ZnCharacterEncoder class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 7/24/2015 15:04'!
iso88591
	^ self newForEncoding: 'iso-8859-1'! !

!ZnCharacterEncoder class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/7/2017 10:21'!
knownEncodingIdentifiers
	"Return a collection of all known encoding identifiers in the system"
	
	self = ZnCharacterEncoder ifFalse: [ ^ #() ].
	^ Array streamContents: [ :all |
		self allSubclassesDo: [ :subClass |
			all nextPutAll: subClass knownEncodingIdentifiers ] ]! !

!ZnCharacterEncoder class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 7/24/2015 15:03'!
latin1
	^ self newForEncoding: 'latin1'! !

!ZnCharacterEncoder class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 6/9/2015 16:54'!
newForEncoding: string
	"Return a new character encoder object for an encoding described by string.
	Search for a subclass that handles it and delegate (subclassResponsibility)."
	
	| concreteSubclass |
	concreteSubclass := self allSubclasses 
		detect: [ :each | each handlesEncoding: string ] 
		ifNone: [ ^ self default ].
	^ concreteSubclass newForEncoding: string! !

!ZnCharacterEncoder class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 7/24/2015 15:04'!
utf8
	^ self newForEncoding: 'utf-8'! !

!ZnCharacterEncoder methodsFor: 'comparing' stamp: 'SvenVanCaekenberghe 12/14/2010 11:56'!
= anObject
	^ self class == anObject class! !

!ZnCharacterEncoder methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 9/15/2014 11:16'!
asZnCharacterEncoder
	^ self! !

!ZnCharacterEncoder methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 2/24/2014 15:37'!
backOnStream: stream
	"Move back one character on stream, assuming stream understands #back"
	
	self subclassResponsibility ! !

!ZnCharacterEncoder methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 12/17/2012 14:47'!
beLenient
	"Don't be strict, which is the default"! !

!ZnCharacterEncoder methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 1/15/2016 12:07'!
decodeAsCodePoints: bytes
	"Decode bytes and return the resulting code points"
	
	| byteStream |
	byteStream := bytes readStream.
	^ Array streamContents: [ :stream |
		[ byteStream atEnd ] whileFalse: [
			stream nextPut: (self nextCodePointFromStream: byteStream) ] ] ! !

!ZnCharacterEncoder methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 12/16/2012 15:17'!
decodeBytes: bytes
	"Decode bytes and return the resulting string"
	
	| byteStream |
	byteStream := bytes readStream.
	^ String streamContents: [ :stream |
		[ byteStream atEnd ] whileFalse: [
			stream nextPut: (self nextFromStream: byteStream) ] ] ! !

!ZnCharacterEncoder methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 1/15/2016 12:11'!
encodeCodePoints: codePoints
	"Encode codePoints and return the resulting byte array"
	
	^ ByteArray streamContents: [ :stream |
		codePoints do: [ :each | 
			self nextPutCodePoint: each toStream: stream ] ]! !

!ZnCharacterEncoder methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 5/21/2013 21:51'!
encodeString: string
	"Encode string and return the resulting byte array"
	
	^ ByteArray streamContents: [ :stream |
		self next: string size putAll: string startingAt: 1 toStream: stream ]! !

!ZnCharacterEncoder methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 1/15/2016 16:03'!
encodedByteCountFor: character
	"Return how many bytes are needed to encode character"
	
	^ self encodedByteCountForCodePoint: character codePoint! !

!ZnCharacterEncoder methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 1/15/2016 16:02'!
encodedByteCountForCodePoint: codePoint
	"Return how many bytes are needed to encode integer code point"
	
	self subclassResponsibility ! !

!ZnCharacterEncoder methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 1/15/2016 16:03'!
encodedByteCountForCodePoints: codePoints
	"Return the exact number of bytes it would take to encode codePoints as a byte array"

	^ codePoints 
		inject: 0 
		into: [ :sum :each |
			sum + (self encodedByteCountForCodePoint: each) ]! !

!ZnCharacterEncoder methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 12/16/2012 15:18'!
encodedByteCountForString: string
	"Return the exact number of bytes it would take to encode string as a byte array"

	^ string 
		inject: 0 
		into: [ :sum :each |
			sum + (self encodedByteCountFor: each) ]! !

!ZnCharacterEncoder methodsFor: 'error handling' stamp: 'SvenVanCaekenberghe 12/16/2012 15:21'!
error: message
	ZnCharacterEncodingError signal: message! !

!ZnCharacterEncoder methodsFor: 'error handling' stamp: 'SvenVanCaekenberghe 11/8/2013 10:44'!
errorOutsideRange
	self error: 'Character Unicode code point outside encoder range'! !

!ZnCharacterEncoder methodsFor: 'comparing' stamp: 'SvenVanCaekenberghe 4/20/2011 12:19'!
hash
	^ self class name hash! !

!ZnCharacterEncoder methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/15/2014 09:47'!
identifier
	^ self subclassResponsibility! !

!ZnCharacterEncoder methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 5/21/2013 21:55'!
next: count putAll: string startingAt: offset toStream: stream
	"Write count characters from string starting at offset to stream."
	
	0 to: count - 1 do: [ :each | 
		self nextPut: (string at: offset + each) toStream: stream ]! !

!ZnCharacterEncoder methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 1/15/2016 12:03'!
nextCodePointFromStream: stream
	"Read and return the next integer code point from stream"
	
	self subclassResponsibility ! !

!ZnCharacterEncoder methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 1/15/2016 12:05'!
nextFromStream: stream
	"Read and return the next character from stream"
	
	| codePoint |
	codePoint := self nextCodePointFromStream: stream.
	^ Character codePoint: codePoint! !

!ZnCharacterEncoder methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 1/15/2016 12:04'!
nextPut: character toStream: stream
	"Write the encoding for character to stream"
	
	self nextPutCodePoint: character codePoint toStream: stream  ! !

!ZnCharacterEncoder methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 1/15/2016 12:03'!
nextPutCodePoint: codePoint toStream: stream
	"Write the encoding for Integer code point to stream"
	
	self subclassResponsibility ! !

!ZnCharacterEncoder methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 5/23/2013 09:25'!
readInto: string startingAt: offset count: requestedCount fromStream: stream
	"Read requestedCount characters into string starting at offset,
	returning the number read, there could be less available when stream is atEnd"

	0 to: requestedCount - 1 do: [ :count |
		stream atEnd ifTrue: [ ^ count ].  
		string at: offset + count put: (self nextFromStream: stream) ].
	^ requestedCount! !

ZnCharacterEncoder subclass: #ZnNullEncoder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-Character-Encoding-Core'!

!ZnNullEncoder commentStamp: 'SvenVanCaekenberghe 6/9/2015 17:08' prior: 0!
I am ZnNullEncoder, a concrete subclass of ZnCharacterEncoder.
I perform no encoding or decoding at all for all characters with a code value below 256.

Note that in principle I could handle Latin1 (ISO-8859-1) or ASCII, although that is not completely correct. To get maximum efficiency, it remains an option.
	
Part of Zinc HTTP Components.!

!ZnNullEncoder class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/7/2017 10:25'!
handlesEncoding: string
	"Return true when my instances handle the encoding described by string.
	Note that in principle I could handle latin1 (iso-8859-1) and ASCII, 
	although that is not completely correct. 
	To get maximum efficiency, it remains an option."
	
	^ (self canonicalEncodingIdentifier: string) = 'null'! !

!ZnNullEncoder class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/7/2017 10:13'!
knownEncodingIdentifiers
	^ #( null )! !

!ZnNullEncoder class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 1/25/2011 11:19'!
newForEncoding: string
	"No further parametrization needed"
	
	^ self new! !

!ZnNullEncoder methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 2/24/2014 15:38'!
backOnStream: stream
	stream back! !

!ZnNullEncoder methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 1/15/2016 16:07'!
decodeBytes: bytes
	"Overwritten for performance reasons"

	^ bytes asString! !

!ZnNullEncoder methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 1/15/2016 16:07'!
encodeString: string
	"Overwritten for performance reasons"

	^ string asByteArray! !

!ZnNullEncoder methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 1/15/2016 16:07'!
encodedByteCountFor: character
	"Overwritten for performance reasons"

	^ 1! !

!ZnNullEncoder methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 1/15/2016 16:04'!
encodedByteCountForCodePoint: codePoint
	^ 1! !

!ZnNullEncoder methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 1/15/2016 16:07'!
encodedByteCountForCodePoints: codePoints
	"Overwritten for performance reasons"

	^ codePoints size! !

!ZnNullEncoder methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 1/15/2016 16:07'!
encodedByteCountForString: string
	"Overwritten for performance reasons"

	^ string size! !

!ZnNullEncoder methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/15/2014 09:49'!
identifier
	^ #ascii! !

!ZnNullEncoder methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 1/15/2016 16:07'!
next: count putAll: string startingAt: offset toStream: stream
	"Write count bytes from string starting at offset to stream."
	"Overwritten for performance reasons"
	
	| bytes |
	bytes := ByteArray new: count.
	bytes replaceFrom: 1 to: count with: string startingAt: offset.
	stream nextPutAll: bytes! !

!ZnNullEncoder methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 1/15/2016 13:11'!
nextCodePointFromStream: stream
	^ stream next! !

!ZnNullEncoder methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 1/15/2016 13:10'!
nextPutCodePoint: codePoint toStream: stream
	codePoint < 256
		ifTrue: [ stream nextPut: codePoint ]
		ifFalse: [ self errorOutsideRange ]! !

!ZnNullEncoder methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 1/15/2016 16:07'!
readInto: string startingAt: offset count: requestedCount fromStream: stream
	"Read requestedCount characters into string starting at offset,
	returning the number read, there could be less available when stream is atEnd"
	"Overwritten for performance reasons"

	| byteBuffer readCount |
	byteBuffer := ByteArray new: requestedCount.
	readCount := stream readInto: byteBuffer startingAt: 1 count: requestedCount.
	string replaceFrom: offset to: offset + readCount - 1 with: byteBuffer startingAt: 1.
	^ readCount! !

ZnCharacterEncoder subclass: #ZnSimplifiedByteEncoder
	instanceVariableNames: 'identifier byteToUnicode unicodeToByte strict'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-Character-Encoding-Core'!

ZnSimplifiedByteEncoder class
	instanceVariableNames: 'byteTextConverters'!

!ZnSimplifiedByteEncoder commentStamp: 'SvenVanCaekenberghe 6/9/2015 17:10' prior: 0!
I am ZnSimplifiedByteEncoder, a concrete subclass of ZnCharacterEncoder.
I handle single byte encodings where byte values 0 to 127 map to ASCII and 128 to 255 are a permutation to Unicode characters.

I am like ZnByteEncoder, a subclass of me, but I implement just two mappings, latin1 or iso-8859-1 and ASCII, to conserve memory.!

ZnSimplifiedByteEncoder class
	instanceVariableNames: 'byteTextConverters'!

ZnSimplifiedByteEncoder subclass: #ZnByteEncoder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-Character-Encoding-Core'!

!ZnByteEncoder commentStamp: 'SvenVanCaekenberghe 9/13/2014 22:45' prior: 0!
I am ZnByteEncoder, a concrete subclass of ZnCharacterEncoder.
I handle single byte encodings where byte values 0 to 127 map to ASCII and 128 to 255 are a permutation to Unicode characters.

I derive my mappings by parsing official unicode.org specifications.

The list of encodings and their names/aliases was taken from http://encoding.spec.whatwg.org/#legacy-single-byte-encodings

I basically support ISO/IEC 8859 1, 2, 3, 4, 5, 6, 7, 8, 10, 13, 14, 15 and 16, Windows Codepages 866, 874, 1250, 1251, 1252, 1253, 1253, 1254, 1255, 1256, 1257, 1258, KOI8 R & U as well as Mac Roman & Cyrillic - each of these with a number of aliases like latin1, latin2, latin3, latin4, latin5, latin6, cyrillic, arabic, greek and hebrew. See #mappingToIdentifiers

Note that most/all of these encodings should be considered legacy, with UTF-8 being the preferred encoding going forward.  

Part of Zinc HTTP Components.!

!ZnByteEncoder class methodsFor: 'mappings' stamp: 'SvenVanCaekenberghe 12/15/2012 19:34'!
cp1250Mapping
	"self generateByteToUnicodeSpec: 'http://unicode.org/Public/MAPPINGS/VENDORS/MICSFT/WINDOWS/CP1250.TXT'"

	^ #(
	16r20AC nil 16r201A nil 16r201E 16r2026 16r2020 16r2021 
	nil 16r2030 16r0160 16r2039 16r015A 16r0164 16r017D 16r0179 
	nil 16r2018 16r2019 16r201C 16r201D 16r2022 16r2013 16r2014 
	nil 16r2122 16r0161 16r203A 16r015B 16r0165 16r017E 16r017A 
	16r00A0 16r02C7 16r02D8 16r0141 16r00A4 16r0104 16r00A6 16r00A7 
	16r00A8 16r00A9 16r015E 16r00AB 16r00AC 16r00AD 16r00AE 16r017B 
	16r00B0 16r00B1 16r02DB 16r0142 16r00B4 16r00B5 16r00B6 16r00B7 
	16r00B8 16r0105 16r015F 16r00BB 16r013D 16r02DD 16r013E 16r017C 
	16r0154 16r00C1 16r00C2 16r0102 16r00C4 16r0139 16r0106 16r00C7 
	16r010C 16r00C9 16r0118 16r00CB 16r011A 16r00CD 16r00CE 16r010E 
	16r0110 16r0143 16r0147 16r00D3 16r00D4 16r0150 16r00D6 16r00D7 
	16r0158 16r016E 16r00DA 16r0170 16r00DC 16r00DD 16r0162 16r00DF 
	16r0155 16r00E1 16r00E2 16r0103 16r00E4 16r013A 16r0107 16r00E7 
	16r010D 16r00E9 16r0119 16r00EB 16r011B 16r00ED 16r00EE 16r010F 
	16r0111 16r0144 16r0148 16r00F3 16r00F4 16r0151 16r00F6 16r00F7 
	16r0159 16r016F 16r00FA 16r0171 16r00FC 16r00FD 16r0163 16r02D9 )! !

!ZnByteEncoder class methodsFor: 'mappings' stamp: 'SvenVanCaekenberghe 9/13/2014 21:11'!
cp1251Mapping
	"self generateByteToUnicodeSpec: 'http://unicode.org/Public/MAPPINGS/VENDORS/MICSFT/WINDOWS/CP1251.TXT'"

	^ #(
	16r0402 16r0403 16r201A 16r0453 16r201E 16r2026 16r2020 16r2021 
	16r20AC 16r2030 16r0409 16r2039 16r040A 16r040C 16r040B 16r040F 
	16r0452 16r2018 16r2019 16r201C 16r201D 16r2022 16r2013 16r2014 
	nil 16r2122 16r0459 16r203A 16r045A 16r045C 16r045B 16r045F 
	16r00A0 16r040E 16r045E 16r0408 16r00A4 16r0490 16r00A6 16r00A7 
	16r0401 16r00A9 16r0404 16r00AB 16r00AC 16r00AD 16r00AE 16r0407 
	16r00B0 16r00B1 16r0406 16r0456 16r0491 16r00B5 16r00B6 16r00B7 
	16r0451 16r2116 16r0454 16r00BB 16r0458 16r0405 16r0455 16r0457 
	16r0410 16r0411 16r0412 16r0413 16r0414 16r0415 16r0416 16r0417 
	16r0418 16r0419 16r041A 16r041B 16r041C 16r041D 16r041E 16r041F 
	16r0420 16r0421 16r0422 16r0423 16r0424 16r0425 16r0426 16r0427 
	16r0428 16r0429 16r042A 16r042B 16r042C 16r042D 16r042E 16r042F 
	16r0430 16r0431 16r0432 16r0433 16r0434 16r0435 16r0436 16r0437 
	16r0438 16r0439 16r043A 16r043B 16r043C 16r043D 16r043E 16r043F 
	16r0440 16r0441 16r0442 16r0443 16r0444 16r0445 16r0446 16r0447 
	16r0448 16r0449 16r044A 16r044B 16r044C 16r044D 16r044E 16r044F )! !

!ZnByteEncoder class methodsFor: 'mappings' stamp: 'SvenVanCaekenberghe 12/15/2012 19:39'!
cp1252Mapping
	"self generateByteToUnicodeSpec: 'http://unicode.org/Public/MAPPINGS/VENDORS/MICSFT/WINDOWS/CP1252.TXT'"

	^ #(
	16r20AC nil 16r201A 16r0192 16r201E 16r2026 16r2020 16r2021 
	16r02C6 16r2030 16r0160 16r2039 16r0152 nil 16r017D nil 
	nil 16r2018 16r2019 16r201C 16r201D 16r2022 16r2013 16r2014 
	16r02DC 16r2122 16r0161 16r203A 16r0153 nil 16r017E 16r0178 
	16r00A0 16r00A1 16r00A2 16r00A3 16r00A4 16r00A5 16r00A6 16r00A7 
	16r00A8 16r00A9 16r00AA 16r00AB 16r00AC 16r00AD 16r00AE 16r00AF 
	16r00B0 16r00B1 16r00B2 16r00B3 16r00B4 16r00B5 16r00B6 16r00B7 
	16r00B8 16r00B9 16r00BA 16r00BB 16r00BC 16r00BD 16r00BE 16r00BF 
	16r00C0 16r00C1 16r00C2 16r00C3 16r00C4 16r00C5 16r00C6 16r00C7 
	16r00C8 16r00C9 16r00CA 16r00CB 16r00CC 16r00CD 16r00CE 16r00CF 
	16r00D0 16r00D1 16r00D2 16r00D3 16r00D4 16r00D5 16r00D6 16r00D7 
	16r00D8 16r00D9 16r00DA 16r00DB 16r00DC 16r00DD 16r00DE 16r00DF 
	16r00E0 16r00E1 16r00E2 16r00E3 16r00E4 16r00E5 16r00E6 16r00E7 
	16r00E8 16r00E9 16r00EA 16r00EB 16r00EC 16r00ED 16r00EE 16r00EF 
	16r00F0 16r00F1 16r00F2 16r00F3 16r00F4 16r00F5 16r00F6 16r00F7 
	16r00F8 16r00F9 16r00FA 16r00FB 16r00FC 16r00FD 16r00FE 16r00FF )! !

!ZnByteEncoder class methodsFor: 'mappings' stamp: 'SvenVanCaekenberghe 12/15/2012 19:35'!
cp1253Mapping
	"self generateByteToUnicodeSpec: 'http://unicode.org/Public/MAPPINGS/VENDORS/MICSFT/WINDOWS/CP1253.TXT'"

	^ #(
	16r20AC nil 16r201A 16r0192 16r201E 16r2026 16r2020 16r2021 
	nil 16r2030 nil 16r2039 nil nil nil nil 
	nil 16r2018 16r2019 16r201C 16r201D 16r2022 16r2013 16r2014 
	nil 16r2122 nil 16r203A nil nil nil nil 
	16r00A0 16r0385 16r0386 16r00A3 16r00A4 16r00A5 16r00A6 16r00A7 
	16r00A8 16r00A9 nil 16r00AB 16r00AC 16r00AD 16r00AE 16r2015 
	16r00B0 16r00B1 16r00B2 16r00B3 16r0384 16r00B5 16r00B6 16r00B7 
	16r0388 16r0389 16r038A 16r00BB 16r038C 16r00BD 16r038E 16r038F 
	16r0390 16r0391 16r0392 16r0393 16r0394 16r0395 16r0396 16r0397 
	16r0398 16r0399 16r039A 16r039B 16r039C 16r039D 16r039E 16r039F 
	16r03A0 16r03A1 nil 16r03A3 16r03A4 16r03A5 16r03A6 16r03A7 
	16r03A8 16r03A9 16r03AA 16r03AB 16r03AC 16r03AD 16r03AE 16r03AF 
	16r03B0 16r03B1 16r03B2 16r03B3 16r03B4 16r03B5 16r03B6 16r03B7 
	16r03B8 16r03B9 16r03BA 16r03BB 16r03BC 16r03BD 16r03BE 16r03BF 
	16r03C0 16r03C1 16r03C2 16r03C3 16r03C4 16r03C5 16r03C6 16r03C7 
	16r03C8 16r03C9 16r03CA 16r03CB 16r03CC 16r03CD 16r03CE nil )! !

!ZnByteEncoder class methodsFor: 'mappings' stamp: 'SvenVanCaekenberghe 9/13/2014 21:11'!
cp1254Mapping
	"self generateByteToUnicodeSpec: 'http://unicode.org/Public/MAPPINGS/VENDORS/MICSFT/WINDOWS/CP1254.TXT'"

	^ #(
	16r20AC nil 16r201A 16r0192 16r201E 16r2026 16r2020 16r2021 
	16r02C6 16r2030 16r0160 16r2039 16r0152 nil nil nil 
	nil 16r2018 16r2019 16r201C 16r201D 16r2022 16r2013 16r2014 
	16r02DC 16r2122 16r0161 16r203A 16r0153 nil nil 16r0178 
	16r00A0 16r00A1 16r00A2 16r00A3 16r00A4 16r00A5 16r00A6 16r00A7 
	16r00A8 16r00A9 16r00AA 16r00AB 16r00AC 16r00AD 16r00AE 16r00AF 
	16r00B0 16r00B1 16r00B2 16r00B3 16r00B4 16r00B5 16r00B6 16r00B7 
	16r00B8 16r00B9 16r00BA 16r00BB 16r00BC 16r00BD 16r00BE 16r00BF 
	16r00C0 16r00C1 16r00C2 16r00C3 16r00C4 16r00C5 16r00C6 16r00C7 
	16r00C8 16r00C9 16r00CA 16r00CB 16r00CC 16r00CD 16r00CE 16r00CF 
	16r011E 16r00D1 16r00D2 16r00D3 16r00D4 16r00D5 16r00D6 16r00D7 
	16r00D8 16r00D9 16r00DA 16r00DB 16r00DC 16r0130 16r015E 16r00DF 
	16r00E0 16r00E1 16r00E2 16r00E3 16r00E4 16r00E5 16r00E6 16r00E7 
	16r00E8 16r00E9 16r00EA 16r00EB 16r00EC 16r00ED 16r00EE 16r00EF 
	16r011F 16r00F1 16r00F2 16r00F3 16r00F4 16r00F5 16r00F6 16r00F7 
	16r00F8 16r00F9 16r00FA 16r00FB 16r00FC 16r0131 16r015F 16r00FF )! !

!ZnByteEncoder class methodsFor: 'mappings' stamp: 'SvenVanCaekenberghe 9/13/2014 21:11'!
cp1255Mapping
	"self generateByteToUnicodeSpec: 'http://unicode.org/Public/MAPPINGS/VENDORS/MICSFT/WINDOWS/CP1255.TXT'"

	^ #(
	16r20AC nil 16r201A 16r0192 16r201E 16r2026 16r2020 16r2021 
	16r02C6 16r2030 nil 16r2039 nil nil nil nil 
	nil 16r2018 16r2019 16r201C 16r201D 16r2022 16r2013 16r2014 
	16r02DC 16r2122 nil 16r203A nil nil nil nil 
	16r00A0 16r00A1 16r00A2 16r00A3 16r20AA 16r00A5 16r00A6 16r00A7 
	16r00A8 16r00A9 16r00D7 16r00AB 16r00AC 16r00AD 16r00AE 16r00AF 
	16r00B0 16r00B1 16r00B2 16r00B3 16r00B4 16r00B5 16r00B6 16r00B7 
	16r00B8 16r00B9 16r00F7 16r00BB 16r00BC 16r00BD 16r00BE 16r00BF 
	16r05B0 16r05B1 16r05B2 16r05B3 16r05B4 16r05B5 16r05B6 16r05B7 
	16r05B8 16r05B9 nil 16r05BB 16r05BC 16r05BD 16r05BE 16r05BF 
	16r05C0 16r05C1 16r05C2 16r05C3 16r05F0 16r05F1 16r05F2 16r05F3 
	16r05F4 nil nil nil nil nil nil nil 
	16r05D0 16r05D1 16r05D2 16r05D3 16r05D4 16r05D5 16r05D6 16r05D7 
	16r05D8 16r05D9 16r05DA 16r05DB 16r05DC 16r05DD 16r05DE 16r05DF 
	16r05E0 16r05E1 16r05E2 16r05E3 16r05E4 16r05E5 16r05E6 16r05E7 
	16r05E8 16r05E9 16r05EA nil nil 16r200E 16r200F nil )! !

!ZnByteEncoder class methodsFor: 'mappings' stamp: 'SvenVanCaekenberghe 9/13/2014 21:12'!
cp1256Mapping
	"self generateByteToUnicodeSpec: 'http://unicode.org/Public/MAPPINGS/VENDORS/MICSFT/WINDOWS/CP1256.TXT'"

	^ #(
	16r20AC 16r067E 16r201A 16r0192 16r201E 16r2026 16r2020 16r2021 
	16r02C6 16r2030 16r0679 16r2039 16r0152 16r0686 16r0698 16r0688 
	16r06AF 16r2018 16r2019 16r201C 16r201D 16r2022 16r2013 16r2014 
	16r06A9 16r2122 16r0691 16r203A 16r0153 16r200C 16r200D 16r06BA 
	16r00A0 16r060C 16r00A2 16r00A3 16r00A4 16r00A5 16r00A6 16r00A7 
	16r00A8 16r00A9 16r06BE 16r00AB 16r00AC 16r00AD 16r00AE 16r00AF 
	16r00B0 16r00B1 16r00B2 16r00B3 16r00B4 16r00B5 16r00B6 16r00B7 
	16r00B8 16r00B9 16r061B 16r00BB 16r00BC 16r00BD 16r00BE 16r061F 
	16r06C1 16r0621 16r0622 16r0623 16r0624 16r0625 16r0626 16r0627 
	16r0628 16r0629 16r062A 16r062B 16r062C 16r062D 16r062E 16r062F 
	16r0630 16r0631 16r0632 16r0633 16r0634 16r0635 16r0636 16r00D7 
	16r0637 16r0638 16r0639 16r063A 16r0640 16r0641 16r0642 16r0643 
	16r00E0 16r0644 16r00E2 16r0645 16r0646 16r0647 16r0648 16r00E7 
	16r00E8 16r00E9 16r00EA 16r00EB 16r0649 16r064A 16r00EE 16r00EF 
	16r064B 16r064C 16r064D 16r064E 16r00F4 16r064F 16r0650 16r00F7 
	16r0651 16r00F9 16r0652 16r00FB 16r00FC 16r200E 16r200F 16r06D2 )! !

!ZnByteEncoder class methodsFor: 'mappings' stamp: 'SvenVanCaekenberghe 9/13/2014 21:12'!
cp1257Mapping
	"self generateByteToUnicodeSpec: 'http://unicode.org/Public/MAPPINGS/VENDORS/MICSFT/WINDOWS/CP1257.TXT'"

	^ #(
	16r20AC nil 16r201A nil 16r201E 16r2026 16r2020 16r2021 
	nil 16r2030 nil 16r2039 nil 16r00A8 16r02C7 16r00B8 
	nil 16r2018 16r2019 16r201C 16r201D 16r2022 16r2013 16r2014 
	nil 16r2122 nil 16r203A nil 16r00AF 16r02DB nil 
	16r00A0 nil 16r00A2 16r00A3 16r00A4 nil 16r00A6 16r00A7 
	16r00D8 16r00A9 16r0156 16r00AB 16r00AC 16r00AD 16r00AE 16r00C6 
	16r00B0 16r00B1 16r00B2 16r00B3 16r00B4 16r00B5 16r00B6 16r00B7 
	16r00F8 16r00B9 16r0157 16r00BB 16r00BC 16r00BD 16r00BE 16r00E6 
	16r0104 16r012E 16r0100 16r0106 16r00C4 16r00C5 16r0118 16r0112 
	16r010C 16r00C9 16r0179 16r0116 16r0122 16r0136 16r012A 16r013B 
	16r0160 16r0143 16r0145 16r00D3 16r014C 16r00D5 16r00D6 16r00D7 
	16r0172 16r0141 16r015A 16r016A 16r00DC 16r017B 16r017D 16r00DF 
	16r0105 16r012F 16r0101 16r0107 16r00E4 16r00E5 16r0119 16r0113 
	16r010D 16r00E9 16r017A 16r0117 16r0123 16r0137 16r012B 16r013C 
	16r0161 16r0144 16r0146 16r00F3 16r014D 16r00F5 16r00F6 16r00F7 
	16r0173 16r0142 16r015B 16r016B 16r00FC 16r017C 16r017E 16r02D9 )! !

!ZnByteEncoder class methodsFor: 'mappings' stamp: 'SvenVanCaekenberghe 9/13/2014 21:12'!
cp1258Mapping
	"self generateByteToUnicodeSpec: 'http://unicode.org/Public/MAPPINGS/VENDORS/MICSFT/WINDOWS/CP1258.TXT'"

	^ #(
	16r20AC nil 16r201A 16r0192 16r201E 16r2026 16r2020 16r2021 
	16r02C6 16r2030 nil 16r2039 16r0152 nil nil nil 
	nil 16r2018 16r2019 16r201C 16r201D 16r2022 16r2013 16r2014 
	16r02DC 16r2122 nil 16r203A 16r0153 nil nil 16r0178 
	16r00A0 16r00A1 16r00A2 16r00A3 16r00A4 16r00A5 16r00A6 16r00A7 
	16r00A8 16r00A9 16r00AA 16r00AB 16r00AC 16r00AD 16r00AE 16r00AF 
	16r00B0 16r00B1 16r00B2 16r00B3 16r00B4 16r00B5 16r00B6 16r00B7 
	16r00B8 16r00B9 16r00BA 16r00BB 16r00BC 16r00BD 16r00BE 16r00BF 
	16r00C0 16r00C1 16r00C2 16r0102 16r00C4 16r00C5 16r00C6 16r00C7 
	16r00C8 16r00C9 16r00CA 16r00CB 16r0300 16r00CD 16r00CE 16r00CF 
	16r0110 16r00D1 16r0309 16r00D3 16r00D4 16r01A0 16r00D6 16r00D7 
	16r00D8 16r00D9 16r00DA 16r00DB 16r00DC 16r01AF 16r0303 16r00DF 
	16r00E0 16r00E1 16r00E2 16r0103 16r00E4 16r00E5 16r00E6 16r00E7 
	16r00E8 16r00E9 16r00EA 16r00EB 16r0301 16r00ED 16r00EE 16r00EF 
	16r0111 16r00F1 16r0323 16r00F3 16r00F4 16r01A1 16r00F6 16r00F7 
	16r00F8 16r00F9 16r00FA 16r00FB 16r00FC 16r01B0 16r20AB 16r00FF )! !

!ZnByteEncoder class methodsFor: 'mappings' stamp: 'SvenVanCaekenberghe 6/10/2015 15:44'!
cp850Mapping
	"This is not included in the MS mappings on the Unicode site, 
	but still in modern use as the default codepage in 
	the Windows Command Shell for multilingual locales"
	"Technically, due to structure of ByteEncoder, it is incomplete, 
	as the lower range corresponds not to ASCII, but to cp437, 
	where character 0 - 31 and 127 have different meanings."
	"See also http://en.wikipedia.org/wiki/Code_page_850"
	
	^ #(
	16r00C7 16r00FC 16r00E9 16r00E2 16r00E4 16r00E0 16r00E5 16r00E7 
	16r00EA 16r00EB 16r00E8 16r00EF 16r00EE 16r00EC 16r00C4 16r00C5 
	16r00C9 16r00E6 16r00C6 16r00F4 16r00F6 16r00F2 16r00FB 16r00F9 
	16r00FF 16r00D6 16r00DC 16r00F8 16r00A3 16r00D8 16r00D7 16r0192 
	16r00E1 16r00ED 16r00F3 16r00FA 16r00F1 16r00D1 16r00AA 16r00BA 
	16r00BF 16r00AE 16r00AC 16r00BD 16r00BC 16r00A1 16r00AB 16r00BB 
	16r2591 16r2592 16r2593 16r2502 16r2524 16r00C1 16r00C2 16r00C0 
	16r00A9 16r2563 16r2551 16r2557 16r255D 16r00A2 16r00A5 16r2510 
	16r2514 16r2534 16r252C 16r251C 16r2500 16r253C 16r00E3 16r00C3 
	16r255A 16r2554 16r2569 16r2566 16r2560 16r2550 16r256C 16r00A4 
	16r00F0 16r00D0 16r00CA 16r00CB 16r00C8 16r0131 16r00CD 16r00CE 
	16r00CF 16r2518 16r250C 16r2588 16r2584 16r00A6 16r00CC 16r2580 
	16r00D3 16r00DF 16r00D4 16r00D2 16r00F5 16r00D5 16r00B5 16r00FE 
	16r00DE 16r00DA 16r00DB 16r00D9 16r00FD 16r00DD 16r00AF 16r00B4 
	16r00AD 16r00B1 16r2017 16r00BE 16r00B6 16r00A7 16r00F7 16r00B8 
	16r00B0 16r00A8 16r00B7 16r00B9 16r00B3 16r00B2 16r25A0 16r00A0 )! !

!ZnByteEncoder class methodsFor: 'mappings' stamp: 'SvenVanCaekenberghe 9/13/2014 21:31'!
cp866Mapping
	"self generateByteToUnicodeSpec: 'http://unicode.org/Public/MAPPINGS/VENDORS/MICSFT/PC/CP866.TXT'"

	^ #(
	16r0410 16r0411 16r0412 16r0413 16r0414 16r0415 16r0416 16r0417 
	16r0418 16r0419 16r041A 16r041B 16r041C 16r041D 16r041E 16r041F 
	16r0420 16r0421 16r0422 16r0423 16r0424 16r0425 16r0426 16r0427 
	16r0428 16r0429 16r042A 16r042B 16r042C 16r042D 16r042E 16r042F 
	16r0430 16r0431 16r0432 16r0433 16r0434 16r0435 16r0436 16r0437 
	16r0438 16r0439 16r043A 16r043B 16r043C 16r043D 16r043E 16r043F 
	16r2591 16r2592 16r2593 16r2502 16r2524 16r2561 16r2562 16r2556 
	16r2555 16r2563 16r2551 16r2557 16r255D 16r255C 16r255B 16r2510 
	16r2514 16r2534 16r252C 16r251C 16r2500 16r253C 16r255E 16r255F 
	16r255A 16r2554 16r2569 16r2566 16r2560 16r2550 16r256C 16r2567 
	16r2568 16r2564 16r2565 16r2559 16r2558 16r2552 16r2553 16r256B 
	16r256A 16r2518 16r250C 16r2588 16r2584 16r258C 16r2590 16r2580 
	16r0440 16r0441 16r0442 16r0443 16r0444 16r0445 16r0446 16r0447 
	16r0448 16r0449 16r044A 16r044B 16r044C 16r044D 16r044E 16r044F 
	16r0401 16r0451 16r0404 16r0454 16r0407 16r0457 16r040E 16r045E 
	16r00B0 16r2219 16r00B7 16r221A 16r2116 16r00A4 16r25A0 16r00A0 )! !

!ZnByteEncoder class methodsFor: 'mappings' stamp: 'SvenVanCaekenberghe 9/13/2014 21:13'!
cp874Mapping
	"self generateByteToUnicodeSpec: 'http://unicode.org/Public/MAPPINGS/VENDORS/MICSFT/WINDOWS/CP874.TXT'"

	^ #(
	16r20AC nil nil nil nil 16r2026 nil nil 
	nil nil nil nil nil nil nil nil 
	nil 16r2018 16r2019 16r201C 16r201D 16r2022 16r2013 16r2014 
	nil nil nil nil nil nil nil nil 
	16r00A0 16r0E01 16r0E02 16r0E03 16r0E04 16r0E05 16r0E06 16r0E07 
	16r0E08 16r0E09 16r0E0A 16r0E0B 16r0E0C 16r0E0D 16r0E0E 16r0E0F 
	16r0E10 16r0E11 16r0E12 16r0E13 16r0E14 16r0E15 16r0E16 16r0E17 
	16r0E18 16r0E19 16r0E1A 16r0E1B 16r0E1C 16r0E1D 16r0E1E 16r0E1F 
	16r0E20 16r0E21 16r0E22 16r0E23 16r0E24 16r0E25 16r0E26 16r0E27 
	16r0E28 16r0E29 16r0E2A 16r0E2B 16r0E2C 16r0E2D 16r0E2E 16r0E2F 
	16r0E30 16r0E31 16r0E32 16r0E33 16r0E34 16r0E35 16r0E36 16r0E37 
	16r0E38 16r0E39 16r0E3A nil nil nil nil 16r0E3F 
	16r0E40 16r0E41 16r0E42 16r0E43 16r0E44 16r0E45 16r0E46 16r0E47 
	16r0E48 16r0E49 16r0E4A 16r0E4B 16r0E4C 16r0E4D 16r0E4E 16r0E4F 
	16r0E50 16r0E51 16r0E52 16r0E53 16r0E54 16r0E55 16r0E56 16r0E57 
	16r0E58 16r0E59 16r0E5A 16r0E5B nil nil nil nil )! !

!ZnByteEncoder class methodsFor: 'utilties' stamp: 'SvenVanCaekenberghe 10/20/2015 16:53'!
generateByteToUnicodeSpec: url
	"Return the formatted source code for an array mapping 
	the top 128 byte to unicode values from a Unicode.org url"
	"self generateByteToUnicodeSpec: 'http://unicode.org/Public/MAPPINGS/ISO8859/8859-2.TXT'."

	| mapping |
	mapping := self parseUnicodeOrgSpec: url.  
	^ String streamContents: [ :stream |
		stream tab; << '"'; << 'self generateByteToUnicodeSpec: '; print: url; << '"'; cr; cr; tab; << '^ #('.
		(self top128FromUnicodeSpec: mapping) doWithIndex: [ :each :index |
			index - 1 \\ 8 = 0 ifTrue: [ stream cr; tab ].
			each isNil
				ifTrue: [ stream print: nil; space ]  
				ifFalse: [ stream << '16r' << (each printPaddedWith: $0 to: 4 base: 16); space ] ].
		stream nextPut: $); cr ]! !

!ZnByteEncoder class methodsFor: 'mappings' stamp: 'SvenVanCaekenberghe 9/13/2014 21:08'!
iso885910Mapping
	"self generateByteToUnicodeSpec: 'http://unicode.org/Public/MAPPINGS/ISO8859/8859-10.TXT'"

	^ #(
	nil nil nil nil nil nil nil nil 
	nil nil nil nil nil nil nil nil 
	nil nil nil nil nil nil nil nil 
	nil nil nil nil nil nil nil nil 
	16r00A0 16r0104 16r0112 16r0122 16r012A 16r0128 16r0136 16r00A7 
	16r013B 16r0110 16r0160 16r0166 16r017D 16r00AD 16r016A 16r014A 
	16r00B0 16r0105 16r0113 16r0123 16r012B 16r0129 16r0137 16r00B7 
	16r013C 16r0111 16r0161 16r0167 16r017E 16r2015 16r016B 16r014B 
	16r0100 16r00C1 16r00C2 16r00C3 16r00C4 16r00C5 16r00C6 16r012E 
	16r010C 16r00C9 16r0118 16r00CB 16r0116 16r00CD 16r00CE 16r00CF 
	16r00D0 16r0145 16r014C 16r00D3 16r00D4 16r00D5 16r00D6 16r0168 
	16r00D8 16r0172 16r00DA 16r00DB 16r00DC 16r00DD 16r00DE 16r00DF 
	16r0101 16r00E1 16r00E2 16r00E3 16r00E4 16r00E5 16r00E6 16r012F 
	16r010D 16r00E9 16r0119 16r00EB 16r0117 16r00ED 16r00EE 16r00EF 
	16r00F0 16r0146 16r014D 16r00F3 16r00F4 16r00F5 16r00F6 16r0169 
	16r00F8 16r0173 16r00FA 16r00FB 16r00FC 16r00FD 16r00FE 16r0138 )! !

!ZnByteEncoder class methodsFor: 'mappings' stamp: 'SvenVanCaekenberghe 9/13/2014 21:08'!
iso885913Mapping
	"self generateByteToUnicodeSpec: 'http://unicode.org/Public/MAPPINGS/ISO8859/8859-13.TXT'"

	^ #(
	nil nil nil nil nil nil nil nil 
	nil nil nil nil nil nil nil nil 
	nil nil nil nil nil nil nil nil 
	nil nil nil nil nil nil nil nil 
	16r00A0 16r201D 16r00A2 16r00A3 16r00A4 16r201E 16r00A6 16r00A7 
	16r00D8 16r00A9 16r0156 16r00AB 16r00AC 16r00AD 16r00AE 16r00C6 
	16r00B0 16r00B1 16r00B2 16r00B3 16r201C 16r00B5 16r00B6 16r00B7 
	16r00F8 16r00B9 16r0157 16r00BB 16r00BC 16r00BD 16r00BE 16r00E6 
	16r0104 16r012E 16r0100 16r0106 16r00C4 16r00C5 16r0118 16r0112 
	16r010C 16r00C9 16r0179 16r0116 16r0122 16r0136 16r012A 16r013B 
	16r0160 16r0143 16r0145 16r00D3 16r014C 16r00D5 16r00D6 16r00D7 
	16r0172 16r0141 16r015A 16r016A 16r00DC 16r017B 16r017D 16r00DF 
	16r0105 16r012F 16r0101 16r0107 16r00E4 16r00E5 16r0119 16r0113 
	16r010D 16r00E9 16r017A 16r0117 16r0123 16r0137 16r012B 16r013C 
	16r0161 16r0144 16r0146 16r00F3 16r014D 16r00F5 16r00F6 16r00F7 
	16r0173 16r0142 16r015B 16r016B 16r00FC 16r017C 16r017E 16r2019 )! !

!ZnByteEncoder class methodsFor: 'mappings' stamp: 'SvenVanCaekenberghe 9/13/2014 21:09'!
iso885914Mapping
	"self generateByteToUnicodeSpec: 'http://unicode.org/Public/MAPPINGS/ISO8859/8859-14.TXT'"

	^ #(
	nil nil nil nil nil nil nil nil 
	nil nil nil nil nil nil nil nil 
	nil nil nil nil nil nil nil nil 
	nil nil nil nil nil nil nil nil 
	16r00A0 16r1E02 16r1E03 16r00A3 16r010A 16r010B 16r1E0A 16r00A7 
	16r1E80 16r00A9 16r1E82 16r1E0B 16r1EF2 16r00AD 16r00AE 16r0178 
	16r1E1E 16r1E1F 16r0120 16r0121 16r1E40 16r1E41 16r00B6 16r1E56 
	16r1E81 16r1E57 16r1E83 16r1E60 16r1EF3 16r1E84 16r1E85 16r1E61 
	16r00C0 16r00C1 16r00C2 16r00C3 16r00C4 16r00C5 16r00C6 16r00C7 
	16r00C8 16r00C9 16r00CA 16r00CB 16r00CC 16r00CD 16r00CE 16r00CF 
	16r0174 16r00D1 16r00D2 16r00D3 16r00D4 16r00D5 16r00D6 16r1E6A 
	16r00D8 16r00D9 16r00DA 16r00DB 16r00DC 16r00DD 16r0176 16r00DF 
	16r00E0 16r00E1 16r00E2 16r00E3 16r00E4 16r00E5 16r00E6 16r00E7 
	16r00E8 16r00E9 16r00EA 16r00EB 16r00EC 16r00ED 16r00EE 16r00EF 
	16r0175 16r00F1 16r00F2 16r00F3 16r00F4 16r00F5 16r00F6 16r1E6B 
	16r00F8 16r00F9 16r00FA 16r00FB 16r00FC 16r00FD 16r0177 16r00FF )! !

!ZnByteEncoder class methodsFor: 'mappings' stamp: 'SvenVanCaekenberghe 12/15/2012 19:37'!
iso885915Mapping
	"self generateByteToUnicodeSpec: 'http://unicode.org/Public/MAPPINGS/ISO8859/8859-15.TXT'"

	^ #(
	nil nil nil nil nil nil nil nil 
	nil nil nil nil nil nil nil nil 
	nil nil nil nil nil nil nil nil 
	nil nil nil nil nil nil nil nil 
	16r00A0 16r00A1 16r00A2 16r00A3 16r20AC 16r00A5 16r0160 16r00A7 
	16r0161 16r00A9 16r00AA 16r00AB 16r00AC 16r00AD 16r00AE 16r00AF 
	16r00B0 16r00B1 16r00B2 16r00B3 16r017D 16r00B5 16r00B6 16r00B7 
	16r017E 16r00B9 16r00BA 16r00BB 16r0152 16r0153 16r0178 16r00BF 
	16r00C0 16r00C1 16r00C2 16r00C3 16r00C4 16r00C5 16r00C6 16r00C7 
	16r00C8 16r00C9 16r00CA 16r00CB 16r00CC 16r00CD 16r00CE 16r00CF 
	16r00D0 16r00D1 16r00D2 16r00D3 16r00D4 16r00D5 16r00D6 16r00D7 
	16r00D8 16r00D9 16r00DA 16r00DB 16r00DC 16r00DD 16r00DE 16r00DF 
	16r00E0 16r00E1 16r00E2 16r00E3 16r00E4 16r00E5 16r00E6 16r00E7 
	16r00E8 16r00E9 16r00EA 16r00EB 16r00EC 16r00ED 16r00EE 16r00EF 
	16r00F0 16r00F1 16r00F2 16r00F3 16r00F4 16r00F5 16r00F6 16r00F7 
	16r00F8 16r00F9 16r00FA 16r00FB 16r00FC 16r00FD 16r00FE 16r00FF )! !

!ZnByteEncoder class methodsFor: 'mappings' stamp: 'SvenVanCaekenberghe 9/13/2014 21:09'!
iso885916Mapping
	"self generateByteToUnicodeSpec: 'http://unicode.org/Public/MAPPINGS/ISO8859/8859-16.TXT'"

	^ #(
	nil nil nil nil nil nil nil nil 
	nil nil nil nil nil nil nil nil 
	nil nil nil nil nil nil nil nil 
	nil nil nil nil nil nil nil nil 
	16r00A0 16r0104 16r0105 16r0141 16r20AC 16r201E 16r0160 16r00A7 
	16r0161 16r00A9 16r0218 16r00AB 16r0179 16r00AD 16r017A 16r017B 
	16r00B0 16r00B1 16r010C 16r0142 16r017D 16r201D 16r00B6 16r00B7 
	16r017E 16r010D 16r0219 16r00BB 16r0152 16r0153 16r0178 16r017C 
	16r00C0 16r00C1 16r00C2 16r0102 16r00C4 16r0106 16r00C6 16r00C7 
	16r00C8 16r00C9 16r00CA 16r00CB 16r00CC 16r00CD 16r00CE 16r00CF 
	16r0110 16r0143 16r00D2 16r00D3 16r00D4 16r0150 16r00D6 16r015A 
	16r0170 16r00D9 16r00DA 16r00DB 16r00DC 16r0118 16r021A 16r00DF 
	16r00E0 16r00E1 16r00E2 16r0103 16r00E4 16r0107 16r00E6 16r00E7 
	16r00E8 16r00E9 16r00EA 16r00EB 16r00EC 16r00ED 16r00EE 16r00EF 
	16r0111 16r0144 16r00F2 16r00F3 16r00F4 16r0151 16r00F6 16r015B 
	16r0171 16r00F9 16r00FA 16r00FB 16r00FC 16r0119 16r021B 16r00FF )! !

!ZnByteEncoder class methodsFor: 'mappings' stamp: 'SvenVanCaekenberghe 12/15/2012 19:36'!
iso88592Mapping
	"self generateByteToUnicodeSpec: 'http://unicode.org/Public/MAPPINGS/ISO8859/8859-2.TXT'"

	^ #(
	nil nil nil nil nil nil nil nil 
	nil nil nil nil nil nil nil nil 
	nil nil nil nil nil nil nil nil 
	nil nil nil nil nil nil nil nil 
	16r00A0 16r0104 16r02D8 16r0141 16r00A4 16r013D 16r015A 16r00A7 
	16r00A8 16r0160 16r015E 16r0164 16r0179 16r00AD 16r017D 16r017B 
	16r00B0 16r0105 16r02DB 16r0142 16r00B4 16r013E 16r015B 16r02C7 
	16r00B8 16r0161 16r015F 16r0165 16r017A 16r02DD 16r017E 16r017C 
	16r0154 16r00C1 16r00C2 16r0102 16r00C4 16r0139 16r0106 16r00C7 
	16r010C 16r00C9 16r0118 16r00CB 16r011A 16r00CD 16r00CE 16r010E 
	16r0110 16r0143 16r0147 16r00D3 16r00D4 16r0150 16r00D6 16r00D7 
	16r0158 16r016E 16r00DA 16r0170 16r00DC 16r00DD 16r0162 16r00DF 
	16r0155 16r00E1 16r00E2 16r0103 16r00E4 16r013A 16r0107 16r00E7 
	16r010D 16r00E9 16r0119 16r00EB 16r011B 16r00ED 16r00EE 16r010F 
	16r0111 16r0144 16r0148 16r00F3 16r00F4 16r0151 16r00F6 16r00F7 
	16r0159 16r016F 16r00FA 16r0171 16r00FC 16r00FD 16r0163 16r02D9 )! !

!ZnByteEncoder class methodsFor: 'mappings' stamp: 'SvenVanCaekenberghe 9/13/2014 21:04'!
iso88593Mapping
	"self generateByteToUnicodeSpec: 'http://unicode.org/Public/MAPPINGS/ISO8859/8859-3.TXT'"

	^ #(
	nil nil nil nil nil nil nil nil 
	nil nil nil nil nil nil nil nil 
	nil nil nil nil nil nil nil nil 
	nil nil nil nil nil nil nil nil 
	16r00A0 16r0126 16r02D8 16r00A3 16r00A4 nil 16r0124 16r00A7 
	16r00A8 16r0130 16r015E 16r011E 16r0134 16r00AD nil 16r017B 
	16r00B0 16r0127 16r00B2 16r00B3 16r00B4 16r00B5 16r0125 16r00B7 
	16r00B8 16r0131 16r015F 16r011F 16r0135 16r00BD nil 16r017C 
	16r00C0 16r00C1 16r00C2 nil 16r00C4 16r010A 16r0108 16r00C7 
	16r00C8 16r00C9 16r00CA 16r00CB 16r00CC 16r00CD 16r00CE 16r00CF 
	nil 16r00D1 16r00D2 16r00D3 16r00D4 16r0120 16r00D6 16r00D7 
	16r011C 16r00D9 16r00DA 16r00DB 16r00DC 16r016C 16r015C 16r00DF 
	16r00E0 16r00E1 16r00E2 nil 16r00E4 16r010B 16r0109 16r00E7 
	16r00E8 16r00E9 16r00EA 16r00EB 16r00EC 16r00ED 16r00EE 16r00EF 
	nil 16r00F1 16r00F2 16r00F3 16r00F4 16r0121 16r00F6 16r00F7 
	16r011D 16r00F9 16r00FA 16r00FB 16r00FC 16r016D 16r015D 16r02D9 )! !

!ZnByteEncoder class methodsFor: 'mappings' stamp: 'SvenVanCaekenberghe 9/13/2014 21:05'!
iso88594Mapping
	"self generateByteToUnicodeSpec: 'http://unicode.org/Public/MAPPINGS/ISO8859/8859-4.TXT'"

	^ #(
	nil nil nil nil nil nil nil nil 
	nil nil nil nil nil nil nil nil 
	nil nil nil nil nil nil nil nil 
	nil nil nil nil nil nil nil nil 
	16r00A0 16r0104 16r0138 16r0156 16r00A4 16r0128 16r013B 16r00A7 
	16r00A8 16r0160 16r0112 16r0122 16r0166 16r00AD 16r017D 16r00AF 
	16r00B0 16r0105 16r02DB 16r0157 16r00B4 16r0129 16r013C 16r02C7 
	16r00B8 16r0161 16r0113 16r0123 16r0167 16r014A 16r017E 16r014B 
	16r0100 16r00C1 16r00C2 16r00C3 16r00C4 16r00C5 16r00C6 16r012E 
	16r010C 16r00C9 16r0118 16r00CB 16r0116 16r00CD 16r00CE 16r012A 
	16r0110 16r0145 16r014C 16r0136 16r00D4 16r00D5 16r00D6 16r00D7 
	16r00D8 16r0172 16r00DA 16r00DB 16r00DC 16r0168 16r016A 16r00DF 
	16r0101 16r00E1 16r00E2 16r00E3 16r00E4 16r00E5 16r00E6 16r012F 
	16r010D 16r00E9 16r0119 16r00EB 16r0117 16r00ED 16r00EE 16r012B 
	16r0111 16r0146 16r014D 16r0137 16r00F4 16r00F5 16r00F6 16r00F7 
	16r00F8 16r0173 16r00FA 16r00FB 16r00FC 16r0169 16r016B 16r02D9 )! !

!ZnByteEncoder class methodsFor: 'mappings' stamp: 'SvenVanCaekenberghe 9/13/2014 21:05'!
iso88595Mapping
	"self generateByteToUnicodeSpec: 'http://unicode.org/Public/MAPPINGS/ISO8859/8859-5.TXT'"

	^ #(
	nil nil nil nil nil nil nil nil 
	nil nil nil nil nil nil nil nil 
	nil nil nil nil nil nil nil nil 
	nil nil nil nil nil nil nil nil 
	16r00A0 16r0401 16r0402 16r0403 16r0404 16r0405 16r0406 16r0407 
	16r0408 16r0409 16r040A 16r040B 16r040C 16r00AD 16r040E 16r040F 
	16r0410 16r0411 16r0412 16r0413 16r0414 16r0415 16r0416 16r0417 
	16r0418 16r0419 16r041A 16r041B 16r041C 16r041D 16r041E 16r041F 
	16r0420 16r0421 16r0422 16r0423 16r0424 16r0425 16r0426 16r0427 
	16r0428 16r0429 16r042A 16r042B 16r042C 16r042D 16r042E 16r042F 
	16r0430 16r0431 16r0432 16r0433 16r0434 16r0435 16r0436 16r0437 
	16r0438 16r0439 16r043A 16r043B 16r043C 16r043D 16r043E 16r043F 
	16r0440 16r0441 16r0442 16r0443 16r0444 16r0445 16r0446 16r0447 
	16r0448 16r0449 16r044A 16r044B 16r044C 16r044D 16r044E 16r044F 
	16r2116 16r0451 16r0452 16r0453 16r0454 16r0455 16r0456 16r0457 
	16r0458 16r0459 16r045A 16r045B 16r045C 16r00A7 16r045E 16r045F )! !

!ZnByteEncoder class methodsFor: 'mappings' stamp: 'SvenVanCaekenberghe 9/13/2014 21:05'!
iso88596Mapping
	"self generateByteToUnicodeSpec: 'http://unicode.org/Public/MAPPINGS/ISO8859/8859-6.TXT'"

	^ #(
	nil nil nil nil nil nil nil nil 
	nil nil nil nil nil nil nil nil 
	nil nil nil nil nil nil nil nil 
	nil nil nil nil nil nil nil nil 
	16r00A0 nil nil nil 16r00A4 nil nil nil 
	nil nil nil nil 16r060C 16r00AD nil nil 
	nil nil nil nil nil nil nil nil 
	nil nil nil 16r061B nil nil nil 16r061F 
	nil 16r0621 16r0622 16r0623 16r0624 16r0625 16r0626 16r0627 
	16r0628 16r0629 16r062A 16r062B 16r062C 16r062D 16r062E 16r062F 
	16r0630 16r0631 16r0632 16r0633 16r0634 16r0635 16r0636 16r0637 
	16r0638 16r0639 16r063A nil nil nil nil nil 
	16r0640 16r0641 16r0642 16r0643 16r0644 16r0645 16r0646 16r0647 
	16r0648 16r0649 16r064A 16r064B 16r064C 16r064D 16r064E 16r064F 
	16r0650 16r0651 16r0652 nil nil nil nil nil 
	nil nil nil nil nil nil nil nil )! !

!ZnByteEncoder class methodsFor: 'mappings' stamp: 'SvenVanCaekenberghe 12/15/2012 19:39'!
iso88597Mapping
	"self generateByteToUnicodeSpec: 'http://unicode.org/Public/MAPPINGS/ISO8859/8859-7.TXT'"

	^ #(
	nil nil nil nil nil nil nil nil 
	nil nil nil nil nil nil nil nil 
	nil nil nil nil nil nil nil nil 
	nil nil nil nil nil nil nil nil 
	16r00A0 16r2018 16r2019 16r00A3 16r20AC 16r20AF 16r00A6 16r00A7 
	16r00A8 16r00A9 16r037A 16r00AB 16r00AC 16r00AD nil 16r2015 
	16r00B0 16r00B1 16r00B2 16r00B3 16r0384 16r0385 16r0386 16r00B7 
	16r0388 16r0389 16r038A 16r00BB 16r038C 16r00BD 16r038E 16r038F 
	16r0390 16r0391 16r0392 16r0393 16r0394 16r0395 16r0396 16r0397 
	16r0398 16r0399 16r039A 16r039B 16r039C 16r039D 16r039E 16r039F 
	16r03A0 16r03A1 nil 16r03A3 16r03A4 16r03A5 16r03A6 16r03A7 
	16r03A8 16r03A9 16r03AA 16r03AB 16r03AC 16r03AD 16r03AE 16r03AF 
	16r03B0 16r03B1 16r03B2 16r03B3 16r03B4 16r03B5 16r03B6 16r03B7 
	16r03B8 16r03B9 16r03BA 16r03BB 16r03BC 16r03BD 16r03BE 16r03BF 
	16r03C0 16r03C1 16r03C2 16r03C3 16r03C4 16r03C5 16r03C6 16r03C7 
	16r03C8 16r03C9 16r03CA 16r03CB 16r03CC 16r03CD 16r03CE nil )! !

!ZnByteEncoder class methodsFor: 'mappings' stamp: 'SvenVanCaekenberghe 9/13/2014 21:07'!
iso88598Mapping
	"self generateByteToUnicodeSpec: 'http://unicode.org/Public/MAPPINGS/ISO8859/8859-8.TXT'"

	^ #(
	nil nil nil nil nil nil nil nil 
	nil nil nil nil nil nil nil nil 
	nil nil nil nil nil nil nil nil 
	nil nil nil nil nil nil nil nil 
	16r00A0 nil 16r00A2 16r00A3 16r00A4 16r00A5 16r00A6 16r00A7 
	16r00A8 16r00A9 16r00D7 16r00AB 16r00AC 16r00AD 16r00AE 16r00AF 
	16r00B0 16r00B1 16r00B2 16r00B3 16r00B4 16r00B5 16r00B6 16r00B7 
	16r00B8 16r00B9 16r00F7 16r00BB 16r00BC 16r00BD 16r00BE nil 
	nil nil nil nil nil nil nil nil 
	nil nil nil nil nil nil nil nil 
	nil nil nil nil nil nil nil nil 
	nil nil nil nil nil nil nil 16r2017 
	16r05D0 16r05D1 16r05D2 16r05D3 16r05D4 16r05D5 16r05D6 16r05D7 
	16r05D8 16r05D9 16r05DA 16r05DB 16r05DC 16r05DD 16r05DE 16r05DF 
	16r05E0 16r05E1 16r05E2 16r05E3 16r05E4 16r05E5 16r05E6 16r05E7 
	16r05E8 16r05E9 16r05EA nil nil 16r200E 16r200F nil )! !

!ZnByteEncoder class methodsFor: 'mappings' stamp: 'SvenVanCaekenberghe 12/15/2012 19:38'!
koi8rMapping
	"self generateByteToUnicodeSpec: 'http://unicode.org/Public/MAPPINGS/VENDORS/MISC/KOI8-R.TXT'"

	^ #(
	16r2500 16r2502 16r250C 16r2510 16r2514 16r2518 16r251C 16r2524 
	16r252C 16r2534 16r253C 16r2580 16r2584 16r2588 16r258C 16r2590 
	16r2591 16r2592 16r2593 16r2320 16r25A0 16r2219 16r221A 16r2248 
	16r2264 16r2265 16r00A0 16r2321 16r00B0 16r00B2 16r00B7 16r00F7 
	16r2550 16r2551 16r2552 16r0451 16r2553 16r2554 16r2555 16r2556 
	16r2557 16r2558 16r2559 16r255A 16r255B 16r255C 16r255D 16r255E 
	16r255F 16r2560 16r2561 16r0401 16r2562 16r2563 16r2564 16r2565 
	16r2566 16r2567 16r2568 16r2569 16r256A 16r256B 16r256C 16r00A9 
	16r044E 16r0430 16r0431 16r0446 16r0434 16r0435 16r0444 16r0433 
	16r0445 16r0438 16r0439 16r043A 16r043B 16r043C 16r043D 16r043E 
	16r043F 16r044F 16r0440 16r0441 16r0442 16r0443 16r0436 16r0432 
	16r044C 16r044B 16r0437 16r0448 16r044D 16r0449 16r0447 16r044A 
	16r042E 16r0410 16r0411 16r0426 16r0414 16r0415 16r0424 16r0413 
	16r0425 16r0418 16r0419 16r041A 16r041B 16r041C 16r041D 16r041E 
	16r041F 16r042F 16r0420 16r0421 16r0422 16r0423 16r0416 16r0412 
	16r042C 16r042B 16r0417 16r0428 16r042D 16r0429 16r0427 16r042A )! !

!ZnByteEncoder class methodsFor: 'mappings' stamp: 'SvenVanCaekenberghe 9/13/2014 21:14'!
koi8uMapping
	"self generateByteToUnicodeSpec: 'http://unicode.org/Public/MAPPINGS/VENDORS/MISC/KOI8-U.TXT'"

	^ #(
	16r2500 16r2502 16r250C 16r2510 16r2514 16r2518 16r251C 16r2524 
	16r252C 16r2534 16r253C 16r2580 16r2584 16r2588 16r258C 16r2590 
	16r2591 16r2592 16r2593 16r2320 16r25A0 16r2219 16r221A 16r2248 
	16r2264 16r2265 16r00A0 16r2321 16r00B0 16r00B2 16r00B7 16r00F7 
	16r2550 16r2551 16r2552 16r0451 16r0454 16r2554 16r0456 16r0457 
	16r2557 16r2558 16r2559 16r255A 16r255B 16r0491 16r255D 16r255E 
	16r255F 16r2560 16r2561 16r0401 16r0404 16r2563 16r0406 16r0407 
	16r2566 16r2567 16r2568 16r2569 16r256A 16r0490 16r256C 16r00A9 
	16r044E 16r0430 16r0431 16r0446 16r0434 16r0435 16r0444 16r0433 
	16r0445 16r0438 16r0439 16r043A 16r043B 16r043C 16r043D 16r043E 
	16r043F 16r044F 16r0440 16r0441 16r0442 16r0443 16r0436 16r0432 
	16r044C 16r044B 16r0437 16r0448 16r044D 16r0449 16r0447 16r044A 
	16r042E 16r0410 16r0411 16r0426 16r0414 16r0415 16r0424 16r0413 
	16r0425 16r0418 16r0419 16r041A 16r041B 16r041C 16r041D 16r041E 
	16r041F 16r042F 16r0420 16r0421 16r0422 16r0423 16r0416 16r0412 
	16r042C 16r042B 16r0417 16r0428 16r042D 16r0429 16r0427 16r042A )! !

!ZnByteEncoder class methodsFor: 'mappings' stamp: 'SvenVanCaekenberghe 9/13/2014 21:32'!
macCyrillicMapping
	"self generateByteToUnicodeSpec: 'http://unicode.org/Public/MAPPINGS/VENDORS/APPLE/CYRILLIC.TXT'"

	^ #(
	16r0410 16r0411 16r0412 16r0413 16r0414 16r0415 16r0416 16r0417 
	16r0418 16r0419 16r041A 16r041B 16r041C 16r041D 16r041E 16r041F 
	16r0420 16r0421 16r0422 16r0423 16r0424 16r0425 16r0426 16r0427 
	16r0428 16r0429 16r042A 16r042B 16r042C 16r042D 16r042E 16r042F 
	16r2020 16r00B0 16r0490 16r00A3 16r00A7 16r2022 16r00B6 16r0406 
	16r00AE 16r00A9 16r2122 16r0402 16r0452 16r2260 16r0403 16r0453 
	16r221E 16r00B1 16r2264 16r2265 16r0456 16r00B5 16r0491 16r0408 
	16r0404 16r0454 16r0407 16r0457 16r0409 16r0459 16r040A 16r045A 
	16r0458 16r0405 16r00AC 16r221A 16r0192 16r2248 16r2206 16r00AB 
	16r00BB 16r2026 16r00A0 16r040B 16r045B 16r040C 16r045C 16r0455 
	16r2013 16r2014 16r201C 16r201D 16r2018 16r2019 16r00F7 16r201E 
	16r040E 16r045E 16r040F 16r045F 16r2116 16r0401 16r0451 16r044F 
	16r0430 16r0431 16r0432 16r0433 16r0434 16r0435 16r0436 16r0437 
	16r0438 16r0439 16r043A 16r043B 16r043C 16r043D 16r043E 16r043F 
	16r0440 16r0441 16r0442 16r0443 16r0444 16r0445 16r0446 16r0447 
	16r0448 16r0449 16r044A 16r044B 16r044C 16r044D 16r044E 16r20AC )! !

!ZnByteEncoder class methodsFor: 'mappings' stamp: 'SvenVanCaekenberghe 12/15/2012 19:38'!
macRomanMapping
	"self generateByteToUnicodeSpec: 'http://unicode.org/Public/MAPPINGS/VENDORS/APPLE/ROMAN.TXT'"

	^ #(
	16r00C4 16r00C5 16r00C7 16r00C9 16r00D1 16r00D6 16r00DC 16r00E1 
	16r00E0 16r00E2 16r00E4 16r00E3 16r00E5 16r00E7 16r00E9 16r00E8 
	16r00EA 16r00EB 16r00ED 16r00EC 16r00EE 16r00EF 16r00F1 16r00F3 
	16r00F2 16r00F4 16r00F6 16r00F5 16r00FA 16r00F9 16r00FB 16r00FC 
	16r2020 16r00B0 16r00A2 16r00A3 16r00A7 16r2022 16r00B6 16r00DF 
	16r00AE 16r00A9 16r2122 16r00B4 16r00A8 16r2260 16r00C6 16r00D8 
	16r221E 16r00B1 16r2264 16r2265 16r00A5 16r00B5 16r2202 16r2211 
	16r220F 16r03C0 16r222B 16r00AA 16r00BA 16r03A9 16r00E6 16r00F8 
	16r00BF 16r00A1 16r00AC 16r221A 16r0192 16r2248 16r2206 16r00AB 
	16r00BB 16r2026 16r00A0 16r00C0 16r00C3 16r00D5 16r0152 16r0153 
	16r2013 16r2014 16r201C 16r201D 16r2018 16r2019 16r00F7 16r25CA 
	16r00FF 16r0178 16r2044 16r20AC 16r2039 16r203A 16rFB01 16rFB02 
	16r2021 16r00B7 16r201A 16r201E 16r2030 16r00C2 16r00CA 16r00C1 
	16r00CB 16r00C8 16r00CD 16r00CE 16r00CF 16r00CC 16r00D3 16r00D4 
	16rF8FF 16r00D2 16r00DA 16r00DB 16r00D9 16r0131 16r02C6 16r02DC 
	16r00AF 16r02D8 16r02D9 16r02DA 16r00B8 16r02DD 16r02DB 16r02C7 )! !

!ZnByteEncoder class methodsFor: 'mappings' stamp: 'SvenVanCaekenberghe 6/10/2015 15:39'!
mappingToIdentifiers
	"Return a dictionay mapping from encoding specifications to a list of encoding names."
	
	^ Dictionary newFromPairs: #(
		"#asciiMapping #('ascii')"
		"#iso88591Mapping #('iso88591' 'latin1')"
		#iso88592Mapping #('iso88592' 'latin2') 
		#iso88593Mapping #('iso88593' 'latin3') 
		#iso88594Mapping #('iso88594' 'latin4') 
		#iso88595Mapping #('iso88595' 'cyrillic') 
		#iso88596Mapping #('iso88596' 'arabic') 
		#iso88597Mapping #('iso88597' 'greek') 
		#iso88598Mapping #('iso88598' 'hebrew') 
		#iso885910Mapping #('iso885910' 'latin6') 
		#iso885913Mapping #('iso885913') 
		#iso885914Mapping #('iso885914') 
		#iso885915Mapping #('iso885915') 
		#iso885916Mapping #('iso885916') 
		#cp1250Mapping #('cp1250' 'windows1250' 'xcp1250') 
		#cp1251Mapping #('cp1251' 'windows1251' 'xcp1251') 
		#cp1252Mapping #('cp1252' 'windows1252' 'xcp1252' 'ibm819')
		#cp1253Mapping #('cp1253' 'windows1253' 'xcp1253')
		#cp1254Mapping #('cp1254' 'windows1254' 'xcp1254' 'iso88599' 'latin5')
		#cp1255Mapping #('cp1255' 'windows1255' 'xcp1255')
		#cp1256Mapping #('cp1256' 'windows1256' 'xcp1256')
		#cp1257Mapping #('cp1257' 'windows1257' 'xcp1257')
		#cp1258Mapping #('cp1258' 'windows1258' 'xcp1258')
		#cp850Mapping #('cp850' 'ibm850' 'oem850' 'doslatin1')
		#cp866Mapping #('cp866' 'ibm866')
		#cp874Mapping #('cp874' 'iso885911' 'windows874' 'dos874')
		#koi8rMapping #('koi8r' 'koi8')
		#koi8uMapping #('koi8u')
		#macRomanMapping #('macroman' 'xmacroman' 'mac' 'macintosh')
		#macCyrillicMapping #('maccyrillic' 'xmaccyrillic') )! !

!ZnByteEncoder class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 10/20/2015 17:04'!
newFromUrl: url
	"Instanciate a new encoder directly from a Unicode.org url"
	"self newFromUrl: 'http://unicode.org/Public/MAPPINGS/ISO8859/8859-2.TXT'"
	"self newFromUrl: 'http://www.unicode.org/Public/MAPPINGS/VENDORS/MICSFT/PC/CP437.TXT'"

	| mapping spec tables |
	mapping := self parseUnicodeOrgSpec: url.
	spec := self top128FromUnicodeSpec: mapping.
	tables := self tablesFromSpec: spec.
	^ self new
		identifier: url;
		byteToUnicode: tables first;
		unicodeToByte: tables second;
		yourself! !

!ZnByteEncoder class methodsFor: 'utilties' stamp: 'SvenVanCaekenberghe 10/20/2015 18:33'!
parseUnicodeOrgSpec: url
	"Parse and return a mapping from byte to unicode values from url."
	"Basic syntax: lines starting with # are comments, else first two fields are read as 0x hex values"
	"self parseUnicodeOrgSpec: 'http://unicode.org/Public/MAPPINGS/ISO8859/8859-2.TXT'."
	
	| mapping |
	mapping := Dictionary new: 256.
	url asZnUrl retrieveContents linesDo: [ :each |
		(each isEmpty or: [ each beginsWith: '#' ])
			ifFalse: [ | tokens hexReader |
				hexReader := [ :string | Integer readFrom: (string readStream skip: 2; yourself) base: 16 ].
				tokens := each findTokens: String tab. 
				(tokens size < 3 or: [ tokens last = '<control>' or: [ tokens last = '#UNDEFINED' ] ]) ifFalse: [
					mapping
						at: (hexReader value: tokens first)
						put: (hexReader value: tokens second) ] ] ].
	^ mapping ! !

!ZnByteEncoder class methodsFor: 'utilties' stamp: 'SvenVanCaekenberghe 10/20/2015 16:49'!
top128FromUnicodeSpec: mapping
	"Return an array mapping the top 128 byte to unicode values from a Unicode.org specification map"

	^ Array new: 128 streamContents: [ :stream |
		128 to: 255 do: [ :each |
			stream nextPut: (mapping at: each ifAbsent: [ nil ]) ] ]! !

!ZnByteEncoder methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 9/15/2014 11:55'!
decodeBytesIntoWideString: bytes
	"Variant of #decodeBytes: that is faster when you know upfront 
	that a WideString is probably needed"

	| byteStream |
	byteStream := bytes readStream.
	^ WideString streamContents: [ :stream |
		[ byteStream atEnd ] whileFalse: [
			stream nextPut: (self nextFromStream: byteStream) ] ] ! !

!ZnSimplifiedByteEncoder class methodsFor: 'mappings' stamp: 'SvenVanCaekenberghe 6/9/2015 17:00'!
asciiMapping
	"ASCII is only defined for the first 127 codes (7-bit)"

	^ #(
	nil nil nil nil nil nil nil nil 
	nil nil nil nil nil nil nil nil 
	nil nil nil nil nil nil nil nil 
	nil nil nil nil nil nil nil nil 
	nil nil nil nil nil nil nil nil 
	nil nil nil nil nil nil nil nil 
	nil nil nil nil nil nil nil nil 
	nil nil nil nil nil nil nil nil 
	nil nil nil nil nil nil nil nil 
	nil nil nil nil nil nil nil nil 
	nil nil nil nil nil nil nil nil 
	nil nil nil nil nil nil nil nil 
	nil nil nil nil nil nil nil nil 
	nil nil nil nil nil nil nil nil 
	nil nil nil nil nil nil nil nil 
	nil nil nil nil nil nil nil nil )! !

!ZnSimplifiedByteEncoder class methodsFor: 'private' stamp: 'SvenVanCaekenberghe 5/22/2017 13:56'!
byteTextConverters
	^ byteTextConverters ifNil: [ self initializeByteTextConverters ]! !

!ZnSimplifiedByteEncoder class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/10/2015 15:41'!
handlesEncoding: string
	"Return true when my instances handle the encoding described by string"
	
	^ self byteTextConverters includesKey: (self canonicalEncodingIdentifier: string)! !

!ZnSimplifiedByteEncoder class methodsFor: 'class initialization' stamp: 'SvenVanCaekenberghe 6/9/2015 16:40'!
initialize
	"Initialize and cache the converters that I know of.
	This includes all of their aliases.
	This method must be changed to make sure it runs 
	when loading in images where it is already present."
	
	self initializeByteTextConverters! !

ZnSimplifiedByteEncoder initialize!

!ZnSimplifiedByteEncoder class methodsFor: 'private' stamp: 'SvenVanCaekenberghe 5/22/2017 13:56'!
initializeByteTextConverters
	"Initialize and cache convertors based on specifications in methods that were autogenerated."

	byteTextConverters := Dictionary new.
	self mappingToIdentifiers
		keysAndValuesDo: [ :mapping :identifiers | 
			| tables |
			tables := self tablesFromSpec: (self perform: mapping).
			identifiers do: [ :each | byteTextConverters at: each put: tables ] ].
	^ byteTextConverters! !

!ZnSimplifiedByteEncoder class methodsFor: 'mappings' stamp: 'SvenVanCaekenberghe 10/20/2015 17:22'!
iso88591Mapping
	"Specification generated by my optional subclass, ZnByteEncoder."
	"ZnByteEncoder generateByteToUnicodeSpec: 'http://unicode.org/Public/MAPPINGS/ISO8859/8859-1.TXT'"

	^ #(
	nil nil nil nil nil nil nil nil 
	nil nil nil nil nil nil nil nil 
	nil nil nil nil nil nil nil nil 
	nil nil nil nil nil nil nil nil 
	16r00A0 16r00A1 16r00A2 16r00A3 16r00A4 16r00A5 16r00A6 16r00A7 
	16r00A8 16r00A9 16r00AA 16r00AB 16r00AC 16r00AD 16r00AE 16r00AF 
	16r00B0 16r00B1 16r00B2 16r00B3 16r00B4 16r00B5 16r00B6 16r00B7 
	16r00B8 16r00B9 16r00BA 16r00BB 16r00BC 16r00BD 16r00BE 16r00BF 
	16r00C0 16r00C1 16r00C2 16r00C3 16r00C4 16r00C5 16r00C6 16r00C7 
	16r00C8 16r00C9 16r00CA 16r00CB 16r00CC 16r00CD 16r00CE 16r00CF 
	16r00D0 16r00D1 16r00D2 16r00D3 16r00D4 16r00D5 16r00D6 16r00D7 
	16r00D8 16r00D9 16r00DA 16r00DB 16r00DC 16r00DD 16r00DE 16r00DF 
	16r00E0 16r00E1 16r00E2 16r00E3 16r00E4 16r00E5 16r00E6 16r00E7 
	16r00E8 16r00E9 16r00EA 16r00EB 16r00EC 16r00ED 16r00EE 16r00EF 
	16r00F0 16r00F1 16r00F2 16r00F3 16r00F4 16r00F5 16r00F6 16r00F7 
	16r00F8 16r00F9 16r00FA 16r00FB 16r00FC 16r00FD 16r00FE 16r00FF )! !

!ZnSimplifiedByteEncoder class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/10/2015 15:42'!
knownEncodingIdentifiers
	^ self byteTextConverters keys! !

!ZnSimplifiedByteEncoder class methodsFor: 'mappings' stamp: 'SvenVanCaekenberghe 6/9/2015 17:05'!
mappingToIdentifiers
	"Return a dictionay mapping from encoding specifications to a list of encoding names."
	
	^ Dictionary newFromPairs: #( 
		#iso88591Mapping #('iso88591' 'latin1')
		#asciiMapping #('ascii') )! !

!ZnSimplifiedByteEncoder class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 5/3/2017 14:17'!
newForEncoding: string
	"Return a new character encoder object for an encoding described by string.
	We use our precomputed ByteTextConverters tables."

	| tables canonicalName |
	canonicalName := self canonicalEncodingIdentifier: string.
	tables := self byteTextConverters at: canonicalName.
	^ self new
		identifier: canonicalName;
		byteToUnicode: tables first;
		unicodeToByte: tables second;
		yourself! !

!ZnSimplifiedByteEncoder class methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/9/2015 16:40'!
tablesFromSpec: mapping
	"Initialize the mappings to and from Unicode based on the 128 element array mapping"

	| byteToUnicode unicodeToByte |
	byteToUnicode := Array new: 128.
	unicodeToByte := Dictionary new.	
	"Mind the offset because first 128 characters are not stored into byteToUnicodeSpec"	
	"Note that some entries are nil"
	mapping
		keysAndValuesDo: [ :index :unicode | 
			unicode ifNotNil: [ 
				byteToUnicode at: index put: (Character value: unicode).
				unicodeToByte at: unicode put: 127 + index ] ].
	^ Array with: byteToUnicode with: unicodeToByte! !

!ZnSimplifiedByteEncoder methodsFor: 'comparing' stamp: 'SvenVanCaekenberghe 5/3/2017 14:18'!
= anObject
	^ super = anObject and: [ self identifier = anObject identifier ]! !

!ZnSimplifiedByteEncoder methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 6/9/2015 16:36'!
backOnStream: stream
	stream back! !

!ZnSimplifiedByteEncoder methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/9/2015 16:36'!
beLenient
	"Don't be strict, which is the default.
	This means that holes in the mapping are let to pass through."

	strict := false! !

!ZnSimplifiedByteEncoder methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/9/2015 16:36'!
byteDomain
	"Return an array with the byte values that are in my domain, that I can decode"
	
	^ (0 to: 127) asArray , ((byteToUnicode withIndexCollect: [ :each :index | each ifNotNil: [ index + 127 ] ]) reject: [ :each | each isNil ]) ! !

!ZnSimplifiedByteEncoder methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/9/2015 16:36'!
byteToUnicode: map
	byteToUnicode := map! !

!ZnSimplifiedByteEncoder methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/9/2015 16:36'!
characterDomain
	"Return a set with the characters that are in my domain, that I can encode"

	^ ((0 to: 127) asSet, unicodeToByte keys) collect: [ :each | Character value: each  ]! !

!ZnSimplifiedByteEncoder methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 1/15/2016 16:07'!
encodedByteCountFor: character
	"Overwritten for performance reasons"
	
	^ 1! !

!ZnSimplifiedByteEncoder methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 1/15/2016 16:05'!
encodedByteCountForCodePoint: character
	^ 1! !

!ZnSimplifiedByteEncoder methodsFor: 'comparing' stamp: 'SvenVanCaekenberghe 5/3/2017 14:10'!
hash
	^ self identifier hash! !

!ZnSimplifiedByteEncoder methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/9/2015 16:36'!
identifier
	^ identifier! !

!ZnSimplifiedByteEncoder methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/9/2015 16:36'!
identifier: object
	identifier := object! !

!ZnSimplifiedByteEncoder methodsFor: 'initialization' stamp: 'SvenVanCaekenberghe 6/9/2015 16:36'!
initialize
	super initialize.
	strict := true! !

!ZnSimplifiedByteEncoder methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 6/9/2015 16:36'!
isLenient
	^ strict not! !

!ZnSimplifiedByteEncoder methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 6/9/2015 16:37'!
isStrict
	^ strict! !

!ZnSimplifiedByteEncoder methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 1/15/2016 13:13'!
nextCodePointFromStream: stream
	"In non-strict mode, we let byte values for holes in our mapping pass through"

	| byteValue |
	^ (byteValue := stream next) < 128
		ifTrue: [ byteValue ]
		ifFalse: [ 
			(byteToUnicode at: byteValue - 127 ifAbsent: [ nil ])
				ifNotNil: [ :unicode | unicode codePoint ]
				ifNil: [ 
					strict
						ifTrue: [ self errorOutsideRange ]
						ifFalse: [ byteValue ] ] ]! !

!ZnSimplifiedByteEncoder methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 1/15/2016 13:15'!
nextFromStream: stream
	"In non-strict mode, we let byte values for holes in our mapping pass through.
	Overwritten for performance reasons"

	| byteValue |
	^ (byteValue := stream next) < 128
		ifTrue: [ Character value: byteValue ]
		ifFalse: [ 
			(byteToUnicode at: byteValue - 127 ifAbsent: [ nil ])
				ifNotNil: [ :unicode | unicode ]
				ifNil: [ 
					strict
						ifTrue: [ self errorOutsideRange ]
						ifFalse: [ Character value: byteValue ] ] ]! !

!ZnSimplifiedByteEncoder methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 1/15/2016 13:14'!
nextPutCodePoint: codePoint toStream: stream
	"In non-strict mode, we let code points for holes in our mapping table pass through"
	
	codePoint < 128
		ifTrue: [ stream nextPut: codePoint ]
		ifFalse: [ 
			| byte |
			byte := unicodeToByte at: codePoint ifAbsent: [ nil ].
			(byte isNil and: [ strict or: [ codePoint > 255 ] ])
				ifTrue: [ self errorOutsideRange ].
			stream nextPut: (byte ifNil: [ codePoint ]) ]! !

!ZnSimplifiedByteEncoder methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 6/9/2015 16:37'!
printOn: stream
	super printOn: stream.
	stream
		nextPut: $(;
		print: identifier.
	strict
		ifTrue: [ stream nextPutAll: ' strict' ].
	stream nextPut: $)! !

!ZnSimplifiedByteEncoder methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/9/2015 16:37'!
unicodeToByte: map
	unicodeToByte := map! !

ZnCharacterEncoder subclass: #ZnUTFEncoder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-Character-Encoding-Core'!

!ZnUTFEncoder commentStamp: 'SvenVanCaekenberghe 1/7/2016 14:44' prior: 0!
I am ZnUTFEncoder. I am a ZnCharacterEncoder. My subclasses deal with the full range of Unicode character code points.!

ZnUTFEncoder subclass: #ZnEndianSensitiveUTFEncoder
	instanceVariableNames: 'endianness'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-Character-Encoding-Core'!

!ZnEndianSensitiveUTFEncoder commentStamp: 'SvenVanCaekenberghe 1/7/2016 13:14' prior: 0!
I am ZnEndianSensitiveUTFEncoder.
I am a ZnCharacterEncoder.
I add support for UTF encodings that are sensitive to endianness.
The default is big endian.!

!ZnEndianSensitiveUTFEncoder class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/7/2016 11:17'!
handlesEncoding: string
	"Return true when my instances handle the encoding described by string"
	
	^ false! !

!ZnEndianSensitiveUTFEncoder class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 1/7/2016 11:17'!
newForEncoding: string
	"Return a new character encoder object for an encoding described by string.
	Try to infer endianness from string, defaulting to big endian."
	
	| encoder |
	encoder := self new.
	(string asLowercase endsWith: 'be') ifTrue: [ encoder beBigEndian ].
	(string asLowercase endsWith: 'le') ifTrue: [ encoder beLittleEndian ].
	^ encoder! !

!ZnEndianSensitiveUTFEncoder methodsFor: 'comparing' stamp: 'SvenVanCaekenberghe 5/3/2017 14:12'!
= anObject
	^ super = anObject and: [ self endianness == anObject endianness ]! !

!ZnEndianSensitiveUTFEncoder methodsFor: 'initialize' stamp: 'SvenVanCaekenberghe 1/7/2016 11:03'!
beBigEndian
	endianness := #big! !

!ZnEndianSensitiveUTFEncoder methodsFor: 'initialize' stamp: 'SvenVanCaekenberghe 1/7/2016 11:03'!
beLittleEndian
	endianness := #little! !

!ZnEndianSensitiveUTFEncoder methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/7/2016 11:03'!
byteOrderMark
	^ 16rFEFF! !

!ZnEndianSensitiveUTFEncoder methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 1/7/2016 11:03'!
encodeStringWithByteOrderMark: string
	"Encode string and return the resulting byte array.
	Always add a Unicode byte order mark (BOM) in front."
	
	^ ByteArray streamContents: [ :stream |
		self nextPutByteOrderMarkToStream: stream.
		self next: string size putAll: string startingAt: 1 toStream: stream ]! !

!ZnEndianSensitiveUTFEncoder methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/3/2017 14:12'!
endianness
	^ endianness! !

!ZnEndianSensitiveUTFEncoder methodsFor: 'comparing' stamp: 'SvenVanCaekenberghe 5/3/2017 14:13'!
hash
	^ super hash bitXor: self endianness hash! !

!ZnEndianSensitiveUTFEncoder methodsFor: 'initialize' stamp: 'SvenVanCaekenberghe 1/7/2016 11:03'!
initialize
	endianness := #big! !

!ZnEndianSensitiveUTFEncoder methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/7/2016 11:03'!
isBigEndian
	^ endianness = #big! !

!ZnEndianSensitiveUTFEncoder methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/7/2016 11:03'!
isLittleEndian
	^ endianness = #little! !

!ZnEndianSensitiveUTFEncoder methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 1/7/2016 11:03'!
nextPutByteOrderMarkToStream: stream
	self write16BitWord: self byteOrderMark toStream: stream ! !

!ZnEndianSensitiveUTFEncoder methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 1/7/2016 11:03'!
printOn: stream
	super printOn: stream.
	stream nextPut: $(.
	stream nextPutAll: endianness; nextPutAll: ' endian'.
	stream nextPut: $)! !

!ZnEndianSensitiveUTFEncoder methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/7/2016 11:03'!
swapEndianness
	self isLittleEndian
		ifTrue: [ self beBigEndian ]
		ifFalse: [ self beLittleEndian ]! !

ZnEndianSensitiveUTFEncoder subclass: #ZnUTF16Encoder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-Character-Encoding-Core'!

!ZnUTF16Encoder commentStamp: '<historical>' prior: 0!
I am ZnUTF16Encoder, a concrete subclass of ZnCharacterEncoder.
I implement the variable length UTF-16 encoding and decoding of Unicode according to RFC 2781.

Wikipedia reference http://en.wikipedia.org/wiki/UTF-16

Part of Zinc HTTP Components.
!

!ZnUTF16Encoder class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/7/2017 10:23'!
handlesEncoding: string
	"Return true when my instances handle the encoding described by string"

	^ self knownEncodingIdentifiers includes: (self canonicalEncodingIdentifier: string)! !

!ZnUTF16Encoder class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/7/2017 10:16'!
knownEncodingIdentifiers
	^ #( 'utf16' 'utf16be' 'utf16le' )! !

!ZnUTF16Encoder methodsFor: 'private' stamp: 'SvenVanCaekenberghe 2/24/2014 16:01'!
back16BitWordOnStream: stream
	| firstByte secondByte |
	firstByte := stream back.
	secondByte := stream back.
	^ self isBigEndian 
		ifTrue: [ secondByte + (firstByte << 8) ]
		ifFalse: [ firstByte + (secondByte << 8) ]! !

!ZnUTF16Encoder methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 2/24/2014 16:02'!
backOnStream: stream
	"Move back one character on stream"

	| word |
	word := self back16BitWordOnStream: stream.
	(word < 16rD800 or: [ word > 16rDBFF ])
		ifFalse: [ 
			self back16BitWordOnStream: stream ]! !

!ZnUTF16Encoder methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 1/15/2016 16:10'!
encodedByteCountForCodePoint: codePoint
	"Return how many bytes are needed to encode integer code point"

	codePoint <= 65535 ifTrue: [ ^ 2 ].
	codePoint <= self maximumUTFCode ifTrue: [ ^ 4 ].
	self errorOutsideRange! !

!ZnUTF16Encoder methodsFor: 'error handling' stamp: 'SvenVanCaekenberghe 11/8/2013 11:40'!
errorIncomplete
	self error: 'Incomplete utf-16 encoding'! !

!ZnUTF16Encoder methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/15/2014 09:49'!
identifier
	^ #utf16! !

!ZnUTF16Encoder methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 1/15/2016 15:18'!
nextCodePointFromStream: stream
	"Read and return the next integer code point from stream"

	| word leadSurrogate trailSurrogate code |
	word := self read16BitWordFromStream: stream.
	(self processByteOrderMark: word)
		ifTrue: [ word := self read16BitWordFromStream: stream ].
	^ (word < 16rD800 or: [ word > 16rDBFF ])
		ifTrue: [
			word ]
		ifFalse: [ 
			leadSurrogate := word.
			trailSurrogate := self read16BitWordFromStream: stream.
			code := (leadSurrogate - 16rD800) * 16r400 + (trailSurrogate - 16rDC00).
			16r10000 + code ]! !

!ZnUTF16Encoder methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 1/15/2016 15:21'!
nextPutCodePoint: codePoint toStream: stream
	"Write the encoding for integer code point to stream"

	| leadSurrogate trailSurrogate shifted |
	(self isSurrogateCodePoint: codePoint) ifTrue: [ self errorOutsideRange ].
	codePoint <= 65535
		ifTrue: [ 
			^ self write16BitWord: codePoint toStream: stream ].
	codePoint <= self maximumUTFCode
		ifTrue: [
			shifted := codePoint - 16r10000.
			leadSurrogate := 16rD800 + (shifted // 16r400).
			trailSurrogate := 16rDC00 + (shifted \\ 16r400).
			self write16BitWord: leadSurrogate toStream: stream.
			self write16BitWord: trailSurrogate toStream: stream ]
		ifFalse: [
			self errorOutsideRange ]! !

!ZnUTF16Encoder methodsFor: 'private' stamp: 'SvenVanCaekenberghe 11/8/2013 15:55'!
processByteOrderMark: word
	^ (word = 16rFEFF or: [ word = 16rFFFE ])
		ifTrue: [ 
			word = 16rFFFE
				ifTrue: [ self swapEndianness ].
			true ]
		ifFalse: [ false ]! !

!ZnUTF16Encoder methodsFor: 'private' stamp: 'SvenVanCaekenberghe 11/8/2013 13:40'!
read16BitWordFromStream: stream
	| firstByte secondByte |
	firstByte := stream next.
	secondByte := stream next.
	(firstByte isNil or: [ secondByte isNil ])
		ifTrue: [ self errorIncomplete ].
	^ self isBigEndian 
		ifTrue: [ secondByte + (firstByte << 8) ]
		ifFalse: [ firstByte + (secondByte << 8) ]
	! !

!ZnUTF16Encoder methodsFor: 'private' stamp: 'SvenVanCaekenberghe 11/8/2013 13:33'!
write16BitWord: word toStream: stream
	self isBigEndian
		ifTrue: [ 
			stream
				nextPut: (word digitAt: 2);
				nextPut: (word digitAt: 1) ]
		ifFalse: [ 
			stream
				nextPut: (word digitAt: 1);
				nextPut: (word digitAt: 2) ]! !

ZnEndianSensitiveUTFEncoder subclass: #ZnUTF32Encoder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-Character-Encoding-Core'!

!ZnUTF32Encoder commentStamp: 'SvenVanCaekenberghe 1/7/2016 14:57' prior: 0!
I am ZnUTF32Encoder, a concrete subclass of ZnCharacterEncoder.
I implement the fixed length UTF-32 encoding and decoding of Unicode according to http://www.unicode.org/versions/Unicode8.0.0/ch03.pdf definitions D90, D99, D100 and D101.

Wikipedia reference http://en.wikipedia.org/wiki/UTF-32

UCS-4 is another name for the same encoding.

Part of Zinc HTTP Components.
!

!ZnUTF32Encoder class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/7/2017 10:23'!
handlesEncoding: string
	"Return true when my instances handle the encoding described by string"

	^ self knownEncodingIdentifiers includes: (self canonicalEncodingIdentifier: string)! !

!ZnUTF32Encoder class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/7/2017 10:16'!
knownEncodingIdentifiers
	^ #( 'utf32' 'utf32be' 'utf32le' 'ucs4' 'ucs4be' 'ucs4le')! !

!ZnUTF32Encoder methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 1/7/2016 11:24'!
backOnStream: stream
	"Move back one character on stream"

	4 timesRepeat: [ stream back ]! !

!ZnUTF32Encoder methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 1/15/2016 16:11'!
encodedByteCountForCodePoint: codePoint
	"Return how many bytes are needed to encode integer codePoint"

	codePoint > self maximumUTFCode 
		ifTrue: [ ^ self errorOutsideRange ].
	^ 4! !

!ZnUTF32Encoder methodsFor: 'error handling' stamp: 'SvenVanCaekenberghe 1/7/2016 11:22'!
errorIncomplete
	self error: 'Incomplete utf-32 encoding'! !

!ZnUTF32Encoder methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/7/2016 11:22'!
identifier
	^ #utf32! !

!ZnUTF32Encoder methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 1/15/2016 15:23'!
nextCodePointFromStream: stream
	"Read and return the next integer code point from stream"

	| codePoint |
	codePoint := self readCodePointFrom: stream.
	(self processByteOrderMark: codePoint)
		ifTrue: [ codePoint := self readCodePointFrom: stream ].
	((self isSurrogateCodePoint: codePoint) or: [ codePoint > self maximumUTFCode ]) 
		ifTrue: [ ^ self errorOutsideRange ].
	^ codePoint! !

!ZnUTF32Encoder methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 1/15/2016 15:24'!
nextPutCodePoint: codePoint toStream: stream
	"Write the encoding for integer code point to stream"

	(self isSurrogateCodePoint: codePoint) 
		ifTrue: [ self errorOutsideRange ].
	codePoint <= self maximumUTFCode
		ifTrue: [
			self writeCodePoint: codePoint to: stream ]
		ifFalse: [
			self errorOutsideRange ]! !

!ZnUTF32Encoder methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/7/2016 14:10'!
processByteOrderMark: codePoint
	^ (codePoint = 16rFEFF or: [ codePoint = 16rFFFE0000 ])
		ifTrue: [ 
			codePoint = 16rFFFE0000
				ifTrue: [ self swapEndianness ].
			true ]
		ifFalse: [ false ]! !

!ZnUTF32Encoder methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/7/2016 13:01'!
readCodePointFrom: stream
	| byte1 byte2 byte3 byte4 |
	byte1 := stream next.
	byte2 := stream next.
	byte3 := stream next.
	byte4 := stream next.
	(byte1 isNil or: [ byte2 isNil or: [ byte3 isNil or: [ byte4 isNil ] ] ])
		ifTrue: [ self errorIncomplete ].
	^ self isBigEndian
		ifTrue: [ 
			(byte1 bitShift: 24) + (byte2 bitShift: 16) + (byte3 bitShift: 8) + byte4 ] 
		ifFalse: [ 
			(byte4 bitShift: 24) + (byte3 bitShift: 16) + (byte2 bitShift: 8) + byte1 ]! !

!ZnUTF32Encoder methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/7/2016 12:55'!
writeCodePoint: codePoint to: stream
	self isBigEndian 
		ifTrue: [ 
			stream 
				nextPut: (codePoint digitAt: 4); 
				nextPut: (codePoint digitAt: 3); 
				nextPut: (codePoint digitAt: 2); 
				nextPut: (codePoint digitAt: 1) ]
		ifFalse: [ 
			stream 
				nextPut: (codePoint digitAt: 1); 
				nextPut: (codePoint digitAt: 2); 
				nextPut: (codePoint digitAt: 3); 
				nextPut: (codePoint digitAt: 4) ]! !

ZnUTFEncoder subclass: #ZnUTF8Encoder
	instanceVariableNames: ''
	classVariableNames: 'ByteASCIISet ByteUTF8Encoding'
	poolDictionaries: ''
	category: 'Zinc-Character-Encoding-Core'!

!ZnUTF8Encoder commentStamp: '<historical>' prior: 0!
I am ZnUTF8Encoder, a concrete subclass of ZnCharacterEncoder.
I implement the variable length UTF-8 encoding and decoding of Unicode according to RFC 3629.

Wikipedia reference http://en.wikipedia.org/wiki/UTF-8

Part of Zinc HTTP Components.!

!ZnUTF8Encoder class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/13/2014 22:10'!
handlesEncoding: string
	"Return true when my instances handle the encoding described by string"
	
	^ (self canonicalEncodingIdentifier: string) = 'utf8'! !

!ZnUTF8Encoder class methodsFor: 'class initialization' stamp: 'SvenVanCaekenberghe 5/25/2013 13:58'!
initialize
	| encoder stream |
	ByteASCIISet := ByteArray new: 256.
	ByteUTF8Encoding := Array new: 256.
	encoder := self new.
	stream := ByteArray new writeStream.
	0 to: 255 do: [ :each |
		| bytes |
		stream reset.
		encoder nextPut: (Character value: each) toStream: stream.
		bytes := stream contents.
		(bytes size = 1 and: [ bytes first = each ])
			ifTrue: [
				ByteASCIISet at: each + 1 put: 0 ]
			ifFalse: [  
				ByteASCIISet at: each + 1 put: 1.
				ByteUTF8Encoding at: each + 1 put: bytes ] ] ! !

ZnUTF8Encoder initialize!

!ZnUTF8Encoder class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/7/2017 10:17'!
knownEncodingIdentifiers
	^ #( utf8 )! !

!ZnUTF8Encoder class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 1/25/2011 11:19'!
newForEncoding: string
	"No further parametrization needed"
	
	^ self new! !

!ZnUTF8Encoder methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 2/24/2014 15:38'!
backOnStream: stream
	[ (stream back bitAnd: 2r11000000) == 2r10000000 ] whileTrue! !

!ZnUTF8Encoder methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 1/15/2016 16:09'!
encodedByteCountForCodePoint: codePoint
	codePoint < 128 ifTrue: [ ^ 1 ].
	codePoint < 2048 ifTrue: [ ^ 2 ].
	codePoint < 65535 ifTrue: [ ^ 3 ].
	codePoint <= self maximumUTFCode ifTrue: [ ^ 4 ].
	self errorOutsideRange! !

!ZnUTF8Encoder methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 11/6/2013 08:32'!
encodingLengthFor: byte
	byte < 128 ifTrue: [ ^ 1 ].
	(byte bitAnd: 2r11100000) == 2r11000000 ifTrue: [ ^ 2 ].
	(byte bitAnd: 2r11110000) == 2r11100000 ifTrue: [ ^ 3 ].
	(byte bitAnd: 2r11111000) == 2r11110000 ifTrue: [ ^ 4 ].
	self errorIllegalLeadingByte! !

!ZnUTF8Encoder methodsFor: 'error handling' stamp: 'SvenVanCaekenberghe 6/3/2013 19:52'!
error: message
	ZnInvalidUTF8 signal: message! !

!ZnUTF8Encoder methodsFor: 'error handling' stamp: 'SvenVanCaekenberghe 11/8/2013 10:52'!
errorIllegalContinuationByte
	self error: 'Illegal continuation byte for utf-8 encoding'! !

!ZnUTF8Encoder methodsFor: 'error handling' stamp: 'SvenVanCaekenberghe 11/6/2013 08:32'!
errorIllegalLeadingByte
	self error: 'Illegal leading byte for utf-8 encoding'! !

!ZnUTF8Encoder methodsFor: 'error handling' stamp: 'SvenVanCaekenberghe 6/3/2013 19:59'!
errorIncomplete
	self error: 'Incomplete utf-8 encoding'! !

!ZnUTF8Encoder methodsFor: 'error handling' stamp: 'SvenVanCaekenberghe 6/4/2013 16:58'!
errorOverlong
	self error: 'Overlong encoding (non-shortest form)'! !

!ZnUTF8Encoder methodsFor: 'private' stamp: 'SvenVanCaekenberghe 11/5/2013 11:35'!
findFirstNonASCIIIn: string startingAt: offset
	"This calls a fast primitive. Note that string can be a ByteString or ByteArray"
	
	offset > string size ifTrue: [ ^ 0 ].
	^ ByteString 
		findFirstInString: string 
		inSet: ByteASCIISet 
		startingAt: offset! !

!ZnUTF8Encoder methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/15/2014 09:49'!
identifier
	^ #utf8! !

!ZnUTF8Encoder methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 5/23/2013 22:51'!
next: count putAll: string startingAt: offset toStream: stream
	"Write count characters from string starting at offset to stream."
	
	string isByteString
		ifTrue: [ self next: count putAllByteString: string startingAt: offset toStream: stream ]
		ifFalse: [ super next: count putAll: string startingAt: offset toStream: stream ]! !

!ZnUTF8Encoder methodsFor: 'private' stamp: 'SvenVanCaekenberghe 5/24/2013 21:12'!
next: count putAllASCII: string startingAt: offset toStream: stream
	"Write count bytes from string starting at offset to stream,
	assuming all characters are in the ASCII set and need no translation"
	
	"For now, we don't use the option of mixing ByteString and ByteArray as similar"
	
	| bytes |
	bytes := ByteArray new: count.
	bytes replaceFrom: 1 to: count with: string startingAt: offset.
	stream nextPutAll: bytes! !

!ZnUTF8Encoder methodsFor: 'private' stamp: 'SvenVanCaekenberghe 5/23/2013 23:05'!
next: count putAllByteString: string startingAt: offset toStream: stream
	"A faster version when string is a ByteString"
	
	| lastIndex nextIndex |
	lastIndex := offset.
	nextIndex := self findFirstNonASCIIIn: string startingAt: lastIndex.
	(nextIndex = 0 or: [ offset + count <= nextIndex ]) 
		ifTrue: [ 
			^ self next: count putAllASCII: string startingAt: offset toStream: stream ].
	[	nextIndex > lastIndex 
			ifTrue: [
				self next: nextIndex - lastIndex putAllASCII: string startingAt: lastIndex toStream: stream ].
		stream nextPutAll: (ByteUTF8Encoding at: (string byteAt: nextIndex) + 1).
		lastIndex := nextIndex + 1.
		nextIndex := self findFirstNonASCIIIn: string startingAt: lastIndex.
		nextIndex = 0 or: [ offset + count <= nextIndex ] ] whileFalse.
	offset + count <= lastIndex 
		ifFalse: [ 
			self next: offset + count - lastIndex putAllASCII: string startingAt: lastIndex toStream: stream ]
	! !

!ZnUTF8Encoder methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 1/15/2016 15:51'!
nextCodePointFromStream: stream
	| code byte next |
	(byte := stream next) < 128
		ifTrue: [ ^ byte ].
	(byte bitAnd: 2r11100000) == 2r11000000
		ifTrue: [ 
			code := byte bitAnd: 2r00011111.
			((next := stream next ifNil: [ self errorIncomplete ]) bitAnd: 2r11000000) == 2r10000000
				ifTrue: [ code := (code bitShift: 6) + (next bitAnd: 2r00111111) ]
				ifFalse: [ ^ self errorIllegalContinuationByte ].
			code < 128 ifTrue: [ self errorOverlong ].
			^ code ].
	(byte bitAnd: 2r11110000) == 2r11100000
		ifTrue: [ 
			code := byte bitAnd: 2r00001111.
			2 timesRepeat: [ 
				((next := stream next ifNil: [ self errorIncomplete ]) bitAnd: 2r11000000) == 2r10000000
					ifTrue: [ code := (code bitShift: 6) + (next bitAnd: 2r00111111) ]
					ifFalse: [ ^ self errorIllegalContinuationByte ] ].
			code < 2048 ifTrue: [ self errorOverlong ].
			(self isSurrogateCodePoint: code) ifTrue: [ self errorOutsideRange ].
			code = 65279 "Unicode Byte Order Mark" ifTrue: [ 
				stream atEnd ifTrue: [ self errorIncomplete ].
				^ self nextCodePointFromStream: stream ].
			^ code ].
	(byte bitAnd: 2r11111000) == 2r11110000
		ifTrue: [ 
			code := byte bitAnd: 2r00000111.
			3 timesRepeat: [ 
				((next := stream next ifNil: [ self errorIncomplete ]) bitAnd: 2r11000000) == 2r10000000
					ifTrue: [ code := (code bitShift: 6) + (next bitAnd: 2r00111111) ]
					ifFalse: [ ^ self errorIllegalContinuationByte ] ].
			code < 65535 ifTrue: [ self errorOverlong ].
			^ code ].
	self errorIllegalLeadingByte! !

!ZnUTF8Encoder methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 1/15/2016 15:13'!
nextPutCodePoint: codePoint toStream: stream
	codePoint < 128 ifTrue: [ 
		^ stream nextPut: codePoint ].
	codePoint < 2048 ifTrue: [ 
		^ stream 
			nextPut: (2r11000000 + (codePoint bitShift: -6)); 
			nextPut: (2r10000000 + (codePoint bitAnd: 2r111111)) ].
	(self isSurrogateCodePoint: codePoint) ifTrue: [ self errorOutsideRange ].
	codePoint < 65536 ifTrue: [ 
		^ stream 
			nextPut: (2r11100000 + (codePoint bitShift: -12));
			nextPut: (2r10000000 + ((codePoint bitShift: -6) bitAnd: 2r111111)); 
			nextPut: (2r10000000 + (codePoint bitAnd: 2r111111)) ].
	codePoint <= self maximumUTFCode ifTrue: [ 
		^ stream 
			nextPut: (2r11110000 + (codePoint bitShift: -18));
			nextPut: (2r10000000 + ((codePoint bitShift: -12) bitAnd: 2r111111));
			nextPut: (2r10000000 + ((codePoint bitShift: -6) bitAnd: 2r111111)); 
			nextPut: (2r10000000 + (codePoint bitAnd: 2r111111)) ].
	self errorOutsideRange
! !

!ZnUTF8Encoder methodsFor: 'private' stamp: 'SvenVanCaekenberghe 11/5/2013 23:22'!
optimizedReadInto: string startingAt: offset count: requestedCount fromStream: stream
	"Read requestedCount characters into string starting at offset,
	returning the number read, there could be less available when stream is atEnd.
	Optimized based on the assumption that most decoded characters will be ASCII.
	I signal a ZnByteStringBecameWideString notification if necessary."
	
	| stringBuffer byteBuffer bytesRead read last next count readStream bytesLeft char |
	stringBuffer := string.
	read := 0.
	"Start by reading the minimum amount of bytes for decoding requestedCount characters"
	byteBuffer := ByteArray new: requestedCount.
	bytesRead := stream readInto: byteBuffer startingAt: 1 count: requestedCount.
	readStream := byteBuffer readStream.
	last := 1.
	next := self findFirstNonASCIIIn: byteBuffer startingAt: last.
	(next = 0 or: [ bytesRead < next ]) 
		ifTrue: [
			"The whole input is ASCII"
			self replace: stringBuffer from: offset to: offset + bytesRead - 1 with: byteBuffer startingAt: 1. 
			^ bytesRead ].
	[	next > last 
			ifTrue: [
				"Copy the ASCII run from last to next"
				count := next - last.
				self replace: stringBuffer from: offset + read to: offset + read + count - 1 with: byteBuffer startingAt: last.
				read := read + count ].
		"Read one UTF-8 encoded non ASCII character"
		readStream position: next - 1.
		count := self encodingLengthFor: readStream peek.
		bytesLeft := bytesRead - next + 1.
		count <= bytesLeft
			ifTrue: [ 
				char := self nextFromStream: readStream ]
			ifFalse: [ | tmpBuffer extraRead |
				tmpBuffer := ByteArray new: count.
				readStream readInto: tmpBuffer startingAt: 1 count: bytesLeft.
				extraRead := stream readInto: tmpBuffer startingAt: bytesLeft + 1 count: count - bytesLeft.
				extraRead = (count - bytesLeft) ifFalse: [ self errorIncomplete ].
				char := self nextFromStream: tmpBuffer readStream ].
		(char isOctetCharacter not and: [ stringBuffer isWideString not ])
			ifTrue: [ stringBuffer := ZnByteStringBecameWideString convert: stringBuffer ].
		stringBuffer at: offset + read put: char.
		last := next + count.
		read := read + 1.
		"Iterate"
		next := self findFirstNonASCIIIn: byteBuffer startingAt: last.
		next = 0 or: [ bytesRead < next ] ] whileFalse.
	bytesRead < last 
		ifFalse: [
			"There is a last ASCII run left" 
			count := bytesRead - last + 1.
			self replace: stringBuffer from: offset + read to: offset + read + count - 1 with: byteBuffer startingAt: last.
			read := read + count ].
	"Try consuming enough input to reach requestedCount since we probably read too little"
	count := requestedCount - read.
	^ read + (self originalReadInto: stringBuffer startingAt: offset + read count: count fromStream: stream)! !

!ZnUTF8Encoder methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/11/2013 15:04'!
originalReadInto: string startingAt: offset count: requestedCount fromStream: stream
	"Read requestedCount characters into string starting at offset,
	returning the number read, there could be less available when stream is atEnd.
	I signal a ZnByteStringBecameWideString notification if necessary"

	| stringBuffer |
	stringBuffer := string.
	0 to: requestedCount - 1 do: [ :count |
		| char |
		stream atEnd ifTrue: [ ^ count ].
		char := self nextFromStream: stream.
		(char isOctetCharacter not and: [ stringBuffer isWideString not ])
			ifTrue: [ stringBuffer := ZnByteStringBecameWideString convert: stringBuffer ].
		stringBuffer at: offset + count put: char ].
	^ requestedCount! !

!ZnUTF8Encoder methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 6/11/2013 15:18'!
readInto: string startingAt: offset count: requestedCount fromStream: stream
	"Read requestedCount characters into string starting at offset,
	returning the number read, there could be less available when stream is atEnd.
	The clean code is in my superclass"

	^ self optimizedReadInto: string startingAt: offset count: requestedCount fromStream: stream! !

!ZnUTF8Encoder methodsFor: 'private' stamp: 'SvenVanCaekenberghe 5/23/2013 22:08'!
replace: string from: start to: stop with: replacement startingAt: repStart
	string isByteString
		ifTrue: [ 
			string replaceFrom: start to: stop with: replacement startingAt: repStart ]
		ifFalse: [ 
			| index repOff |
			repOff := repStart - start.
			index := start - 1.
			[ (index := index + 1) <= stop ] 
				whileTrue: [ 
					string at: index put: (replacement at: repOff + index) asCharacter ] ]! !

!ZnUTFEncoder class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/7/2016 14:44'!
handlesEncoding: string
	"Return true when my instances handle the encoding described by string"
	
	^ false! !

!ZnUTFEncoder methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 1/7/2016 14:45'!
decodeBytes: bytes
	"Overridden to prevent the automagic switch from ByteString to WideString 
	and the slow #becomeForward: in there. See also #decodeBytesIntoWideString:"
	
	| byteStream |
	byteStream := bytes readStream.
	^ String streamContents: [ :stream |
		[ byteStream atEnd ] whileFalse: [ | char |
			char := self nextFromStream: byteStream.
			(char isOctetCharacter not and: [ stream originalContents isWideString not ])
				ifTrue: [ | wideString position |
					position := stream position.
					wideString := WideString from: stream originalContents.
					stream on: wideString; setFrom: position + 1 to: position ].
			stream nextPut: char ] ] ! !

!ZnUTFEncoder methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 1/7/2016 14:46'!
decodeBytesIntoWideString: bytes
	"Variant of #decodeBytes: that is faster when you know upfront 
	that a WideString is probably needed"

	| byteStream |
	byteStream := bytes readStream.
	^ WideString streamContents: [ :stream |
		[ byteStream atEnd ] whileFalse: [
			stream nextPut: (self nextFromStream: byteStream) ] ] ! !

!ZnUTFEncoder methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/15/2016 10:13'!
isSurrogateCodePoint: codePoint
	"Surrogate Code Points should not be encoded or decoded because they are not Unicode scalar values"
	
	^ codePoint between: 16rD800 and: 16rDFFF! !

!ZnUTFEncoder methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/7/2016 14:46'!
maximumUTFCode
	^ 16r10FFFF! !

Object subclass: #ZnEncodedStream
	instanceVariableNames: 'stream encoder'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-Character-Encoding-Core'!

!ZnEncodedStream commentStamp: 'SvenVanCaekenberghe 1/26/2016 10:17' prior: 0!
I am ZnEncodedStream, an abstract support class for read and write streams on an encoded binary stream.

Part of Zinc HTTP Components!

ZnEncodedStream subclass: #ZnEncodedReadStream
	instanceVariableNames: 'peeked'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-Character-Encoding-Core'!

!ZnEncodedReadStream commentStamp: 'SvenVanCaekenberghe 1/26/2016 10:20' prior: 0!
I am ZnEncodedReadStream, an abstract support class for read streams on a binary encoded wrapped stream.

Part of Zinc HTTP Components!

ZnEncodedReadStream subclass: #ZnCharacterReadStream
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-Character-Encoding-Core'!

!ZnCharacterReadStream commentStamp: '<historical>' prior: 0!
I am ZnCharacterReadStream.
I wrap another binary ReadStream and use a ZnCharacerEncoder to allow Characters to be read.

I am not positionable, but I do allow a one character peek using a one character internal buffer.

Part of Zinc HTTP Components.
!

!ZnCharacterReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/3/2012 15:09'!
collectionSpecies
	^ String! !

!ZnCharacterReadStream methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/26/2016 11:12'!
nextElement
	^ self encoder nextFromStream: stream! !

!ZnCharacterReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/26/2016 11:14'!
nextLine
	"Read a CR, LF or CRLF terminated line, returning the contents of the line without the EOL. Return nil when the receiver is #atEnd."
	
	self atEnd ifTrue: [ ^ nil ].
	^ self collectionSpecies streamContents: [ :out | | eol char |
		eol := false.
		[ eol ] whileFalse: [ 
			char := self next.
			(char isNil or: [ char = Character lf ])
				ifTrue: [ eol := true ]
				ifFalse: [ 
					char = Character cr 
						ifTrue: [ eol := true. self peekFor: Character lf ]
						ifFalse: [ out nextPut: char ] ] ] ]! !

!ZnCharacterReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/26/2016 10:52'!
readInto: collection startingAt: offset count: requestedCount 
	"Read count elements and place them in collection starting at offset.
	Return the number of elements actually read."
	
	^ peeked 
		ifNil: [ | readCount |
			[ readCount := self encoder 
					readInto: collection 
					startingAt: offset 
					count: requestedCount 
					fromStream: stream ]
				on: ZnByteStringBecameWideString 
				do: [ :byteStringBecameWideString | 
					byteStringBecameWideString becomeForward; resume ].
			readCount ]
		ifNotNil: [ 
			collection at: offset put: peeked.
			peeked := nil.
			(self 
				readInto: collection 
				startingAt: offset + 1
				count: requestedCount - 1) + 1 ]! !

ZnEncodedReadStream subclass: #ZnCodePointReadStream
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-Character-Encoding-Core'!

!ZnCodePointReadStream commentStamp: 'SvenVanCaekenberghe 1/26/2016 11:09' prior: 0!
I am ZnCodePointReadStream.
I wrap another binary ReadStream and use a ZnCharacerEncoder to allow Integer code points to be read.

I am not positionable, but I do allow a one code point peek using a one code point internal buffer.

Part of Zinc HTTP Components.
!

!ZnCodePointReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/26/2016 11:10'!
collectionSpecies
	^ Array! !

!ZnCodePointReadStream methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/26/2016 11:28'!
nextElement
	^ self encoder nextCodePointFromStream: stream! !

!ZnEncodedReadStream methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/26/2016 10:52'!
atEnd
	^ peeked isNil and: [ stream atEnd ]! !

!ZnEncodedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/26/2016 10:49'!
contents
	"This is technically not correct, but it is better than nothing"
	
	^ self upToEnd! !

!ZnEncodedReadStream methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/26/2016 10:49'!
isBinary
	^ false! !

!ZnEncodedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/26/2016 11:13'!
next
	^ peeked
		ifNil: [ 
			stream atEnd ifFalse: [ self nextElement ] ]
		ifNotNil: [ | character |
			character := peeked.
			peeked := nil.
			character ]! !

!ZnEncodedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/26/2016 10:50'!
next: requestedCount 
	"Read requestedCount elements into new collection and return it,
	 it could be that less elements were available"

	^ self 
		next: requestedCount 
		into: (self collectionSpecies new: requestedCount)! !

!ZnEncodedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/26/2016 10:50'!
next: requestedCount into: collection
	"Read requestedCount elements into collection,
	returning a copy if less elements are available"
	
	^ self
		next: requestedCount
		into: collection
		startingAt: 1! !

!ZnEncodedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/26/2016 10:50'!
next: requestedCount into: collection startingAt: offset
	"Read requestedCount elements into collection starting at offset,
	returning a copy if less elements are available"
	
	| readCount |
	readCount := self 
		readInto: collection 
		startingAt: offset 
		count: requestedCount.
	^ requestedCount = readCount
		ifTrue: [ collection ]
		ifFalse: [ collection copyFrom: 1 to: offset + readCount - 1 ]! !

!ZnEncodedReadStream methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/26/2016 11:12'!
nextElement
	self subclassResponsibility ! !

!ZnEncodedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/26/2016 11:13'!
peek
	^ peeked
		ifNil: [
			stream atEnd ifFalse: [ peeked := self nextElement ] ]! !

!ZnEncodedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/26/2016 10:50'!
peekFor: object
	^ self peek = object
		ifTrue: [ 
			self next.
			true ]
		ifFalse: [ false ]! !

!ZnEncodedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/26/2016 11:00'!
readInto: collection startingAt: offset count: requestedCount
	"Read requestedCount elements into collection starting at offset,
	returning the number of elements read, there could be less elements available.
	This is an inefficient abstract implementation, reading one by one."
	
	0 to: requestedCount - 1 do: [ :count | | object |
		(object := self next) ifNil: [ ^ count ].  
		collection at: offset + count put: object ].
	^ requestedCount! !

!ZnEncodedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/26/2016 10:50'!
skip: count
	count timesRepeat: [ self next ]! !

!ZnEncodedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/26/2016 10:50'!
upTo: anObject 	
	^ self collectionSpecies 
		streamContents: [ :out | | element |
			[ self atEnd or: [ (element := self next) = anObject ] ] whileFalse: [ 
				out nextPut: element ] ]! !

!ZnEncodedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/26/2016 10:51'!
upToEnd
	^ self collectionSpecies
		streamContents: [ :collectionStream | 
			[ self atEnd ] whileFalse: [ collectionStream nextPut: self next ] ]! !

!ZnEncodedStream class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/26/2016 10:23'!
defaultEncoder
	^ ZnUTF8Encoder new! !

!ZnEncodedStream class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 1/26/2016 10:24'!
on: wrappedStream
	^ self new
		on: wrappedStream;
		yourself! !

!ZnEncodedStream class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 1/26/2016 10:24'!
on: wrappedStream encoding: encoding
	^ self new
		on: wrappedStream;
		encoding: encoding;
		yourself! !

!ZnEncodedStream methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 1/26/2016 10:22'!
close
	stream close! !

!ZnEncodedStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/26/2016 10:22'!
encoder
	^ encoder ifNil: [ encoder := self class defaultEncoder ]! !

!ZnEncodedStream methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 1/26/2016 10:22'!
encoder: characterEncoder
	encoder := characterEncoder! !

!ZnEncodedStream methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 1/26/2016 10:22'!
encoding: encoding
	encoder := encoding asZnCharacterEncoder! !

!ZnEncodedStream methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 1/26/2016 10:22'!
on: wrappedStream
	stream := wrappedStream! !

!ZnEncodedStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/26/2016 10:22'!
wrappedStream
	^ stream! !

ZnEncodedStream subclass: #ZnEncodedWriteStream
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-Character-Encoding-Core'!

!ZnEncodedWriteStream commentStamp: 'SvenVanCaekenberghe 1/26/2016 10:21' prior: 0!
I am ZnEncodedWriteStream, an abstract support class for write streams on a binary encoded wrapped stream.

Part of Zinc HTTP Components!

ZnEncodedWriteStream subclass: #ZnCharacterWriteStream
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-Character-Encoding-Core'!

!ZnCharacterWriteStream commentStamp: '<historical>' prior: 0!
I am ZnCharacterWriteStream.
I wrap another binary WriteStream and use a ZnCharacerEncoder to allow Characters to be written.

Part of Zinc HTTP Components.!

!ZnCharacterWriteStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/16/2017 10:33'!
cr
	self nextPut: Character cr! !

!ZnCharacterWriteStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/19/2014 13:13'!
crlf
	self cr; lf! !

!ZnCharacterWriteStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/19/2014 13:13'!
lf
	self nextPut: Character lf! !

!ZnCharacterWriteStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 11/4/2013 15:25'!
next: count putAll: collection startingAt: offset
	"Write count characters from collection starting at offset."
	
	self encoder 
		next: count 
		putAll: collection 
		startingAt: offset 
		toStream: stream ! !

!ZnCharacterWriteStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/3/2012 15:56'!
nextPut: object
	self encoder 
		nextPut: object 
		toStream: stream.
	^ object! !

!ZnCharacterWriteStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 10/3/2014 10:11'!
print: object
	object printOn: self! !

!ZnCharacterWriteStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/3/2012 15:52'!
space
	self nextPut: Character space! !

!ZnCharacterWriteStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 10/3/2014 10:03'!
tab
	self nextPut: Character tab! !

ZnEncodedWriteStream subclass: #ZnCodePointWriteStream
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-Character-Encoding-Core'!

!ZnCodePointWriteStream commentStamp: 'SvenVanCaekenberghe 1/26/2016 11:08' prior: 0!
I am ZnCodePointWriteStream.
I wrap another binary WriteStream and use a ZnCharacerEncoder to allow Integer code points to be written.

Part of Zinc HTTP Components.!

!ZnCodePointWriteStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/26/2016 11:16'!
nextPut: object
	self encoder 
		nextPutCodePoint: object 
		toStream: stream.
	^ object! !

!ZnEncodedWriteStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/26/2016 10:52'!
<< collection
	^ self nextPutAll: collection! !

!ZnEncodedWriteStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/26/2016 10:52'!
flush
	stream flush! !

!ZnEncodedWriteStream methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/26/2016 10:52'!
isBinary
	^ false! !

!ZnEncodedWriteStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/26/2016 10:52'!
next: count putAll: collection	
	self 
		next: count 
		putAll: collection 
		startingAt: 1! !

!ZnEncodedWriteStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/26/2016 10:57'!
next: count putAll: collection startingAt: offset
	"Write count items from collection starting at offset.
	This is an inefficient abstract implementation writing one by one."
	
	0 to: count - 1 do: [ :each | 
		self nextPut: (collection at: offset + each) ]! !

!ZnEncodedWriteStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/26/2016 10:58'!
nextPut: anObject
	self subclassResponsibility! !

!ZnEncodedWriteStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/26/2016 10:53'!
nextPutAll: collection
	self 
		next: collection size 
		putAll: collection 
		startingAt: 1! !

Object subclass: #ZnFastLineReader
	instanceVariableNames: 'readStream cr lf bufferStream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-Character-Encoding-Core'!

!ZnFastLineReader commentStamp: 'SvenVanCaekenberghe 12/11/2014 14:10' prior: 0!
I am ZnFastLineReader, a helper to efficiently read CR, LF or CRLF terminated lines from a character stream.!

!ZnFastLineReader class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 12/11/2014 13:37'!
on: characterReadStream
	^ self new 
		on: characterReadStream;
		yourself! !

!ZnFastLineReader methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/11/2014 13:31'!
atEnd
	^ readStream atEnd! !

!ZnFastLineReader methodsFor: 'initialize' stamp: 'SvenVanCaekenberghe 12/11/2014 14:09'!
beWide
	self bufferStream: (WideString new: 32) writeStream! !

!ZnFastLineReader methodsFor: 'initialize' stamp: 'SvenVanCaekenberghe 12/11/2014 14:09'!
bufferStream: characterWriteStream
	^ bufferStream := characterWriteStream! !

!ZnFastLineReader methodsFor: 'initialize' stamp: 'SvenVanCaekenberghe 12/11/2014 13:31'!
close
	readStream close! !

!ZnFastLineReader methodsFor: 'initialize' stamp: 'SvenVanCaekenberghe 12/11/2014 13:47'!
initialize
	super initialize.
	cr := Character cr.
	lf := Character lf! !

!ZnFastLineReader methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/11/2014 13:50'!
linesDo: block
	[ self atEnd ]
		whileFalse: [ 
			| line |
			line := self nextLine.
			line isNil
				ifFalse: [ block value: line ] ]! !

!ZnFastLineReader methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/11/2014 13:55'!
nextLine
	"Read a CR, LF or CRLF terminated line, returning the contents of the line without the EOL. Return nil when the receiver is #atEnd."
	
	self atEnd ifTrue: [ ^ nil ].
	^ self streamContents: [ :out | | eol char |
		eol := false.
		[ eol ] whileFalse: [ 
			char := readStream next.
			(char isNil or: [ char == lf ])
				ifTrue: [ eol := true ]
				ifFalse: [ 
					char == cr 
						ifTrue: [ eol := true. readStream peekFor: lf ]
						ifFalse: [  out nextPut: char ] ] ] ]! !

!ZnFastLineReader methodsFor: 'initialize' stamp: 'SvenVanCaekenberghe 12/11/2014 13:31'!
on: characterReadStream
	readStream := characterReadStream ! !

!ZnFastLineReader methodsFor: 'private' stamp: 'SvenVanCaekenberghe 12/11/2014 14:07'!
streamContents: block
	"Like readStream collectionSpecies streamContents: block
	but reusing the underlying buffer for improved efficiency"
	
	bufferStream 
		ifNil: [ 
			bufferStream := (readStream collectionSpecies new: 32) writeStream ].
	bufferStream reset.
	block value: bufferStream.
	^ bufferStream contents! !

Object subclass: #ZnPercentEncoder
	instanceVariableNames: 'characterEncoder safeSet decodePlusAsSpace'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-Character-Encoding-Core'!

!ZnPercentEncoder commentStamp: 'SvenVanCaekenberghe 2/14/2014 14:06' prior: 0!
I am ZnPercentEncoder.
I implement RFC 3986 percent encoding and decoding.

All characters that are not part of a safe set are encoded using a percent (%) followed by a two digit hexadecimal number of a byte value. Non-ASCII characters are first encoded, normally using UTF-8.

My #encode: and #decode: messages work from String to String.

My decoder will accept + as an encoding for a space by default.

See also http://en.wikipedia.org/wiki/Percent-encoding

Part of Zinc HTTP Components
!

!ZnPercentEncoder class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 10/23/2013 18:55'!
rfc3986UnreservedCharacters
	"Return the unreserved characters according to RFC 3986 section 2.3.
	This is the most narrow safe set to be used in a better safe than sorry approach."

	^ 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789-_.~'! !

!ZnPercentEncoder methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/10/2017 17:14'!
characterEncoder
	"Return the character encoder that I currently use.
	If not set, I will default to using UTF-8."
	
	^ characterEncoder ifNil: [ characterEncoder := ZnDefaultCharacterEncoder value ]! !

!ZnPercentEncoder methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 12/15/2012 13:53'!
characterEncoder: object
	"Set the character encoding to use to object."
	
	characterEncoder := object! !

!ZnPercentEncoder methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 2/14/2014 14:23'!
decode: string
	"Decode a percent encoded string and return the resulting byte array.
	After percent decoding I will use my character encoder to convert byte values to characters."

	| bytes stringStream |
	stringStream := string readStream.
	bytes := ByteArray streamContents: [ :byteStream | 
		self decode: stringStream to: byteStream ].
	^ self characterEncoder decodeBytes: bytes! !

!ZnPercentEncoder methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 2/14/2014 14:20'!
decode: stringStream to: byteStream
	| char |
	self decodePlusAsSpace.
	[ stringStream atEnd ]
		whileFalse: [ 
			((char := stringStream next) == $+ and: [ decodePlusAsSpace ])
				ifTrue: [ byteStream nextPut: 32 ]
				ifFalse: [ 
					char == $%
						ifTrue: [ byteStream nextPut: (self readHexFrom: stringStream) ]
						ifFalse: [ 
							char charCode < 128
								ifTrue: [ byteStream nextPut: char charCode ]
								ifFalse: [ self errorAsciiCharacterExpected ] ] ] ]! !

!ZnPercentEncoder methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/14/2014 14:08'!
decodePlusAsSpace
	"Return if $+ on input should be decoded as Character space.
	This is normally only done application/x-www-form-urlencoded data,
	but is is on by default anyway."
	
	^ decodePlusAsSpace ifNil: [ decodePlusAsSpace := true ]! !

!ZnPercentEncoder methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 2/14/2014 14:10'!
decodePlusAsSpace: boolean
	"When boolean is true, $+ on input will be decoded as Character space.
	Else $+ is treated as a normal character, filtered by the safe set.
	This is normally only done application/x-www-form-urlencoded data,
	but is is on by default anyway."
	
	decodePlusAsSpace := boolean! !

!ZnPercentEncoder methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 2/14/2014 14:24'!
encode: string
	"Encode string using percent encoding and return the resulting string.
	I will use my character encoder to convert string to bytes and then
	percent encode all byte values that are not in my safe set."

	^ String streamContents: [ :stream | 
		self encode: string readStream to: stream ]! !

!ZnPercentEncoder methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 10/23/2013 18:57'!
encode: readStream to: writeStream
	| bytes buffer byte |
	buffer := (bytes := ByteArray new: 4) writeStream.
	self safeSet; characterEncoder.
	[ readStream atEnd ]
		whileFalse: [ 
			buffer reset.
			characterEncoder nextPut: readStream next toStream: buffer.
			1 to: buffer position do: [ :index | 
				(safeSet includes: (byte := bytes at: index))
					ifTrue: [ writeStream nextPut: byte asCharacter ]
					ifFalse: [ 
						writeStream nextPut: $%.
						self writeHex: byte to: writeStream ] ] ]! !

!ZnPercentEncoder methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 2/14/2014 14:24'!
errorAsciiCharacterExpected
	ZnCharacterEncodingError signal: 'ASCII character expected'! !

!ZnPercentEncoder methodsFor: 'error handling' stamp: 'SvenVanCaekenberghe 2/14/2014 14:12'!
errorHexDigitExpected
	ZnCharacterEncodingError signal: 'hex digit expected'! !

!ZnPercentEncoder methodsFor: 'private' stamp: 'SvenVanCaekenberghe 2/14/2014 14:12'!
readHexFrom: stream
	| first second |
	(stream atEnd not and: [ (first := stream next digitValue) between: 0 and: 15 ])
		ifFalse: [ self errorHexDigitExpected ].
	(stream atEnd not and: [ (second := stream next digitValue) between: 0 and: 15 ])
		ifFalse: [ self errorHexDigitExpected ].
	^ (first << 4) + second! !

!ZnPercentEncoder methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 10/23/2013 18:55'!
safeSet
	"Return the safe set of characters that I will not encode, as a byte array.
	If not set, I will default to the most commonly used safe set"
	
	^ safeSet ifNil: [ safeSet := self class rfc3986UnreservedCharacters asByteArray ]! !

!ZnPercentEncoder methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 12/15/2012 13:58'!
safeSet: string
	"Set my safe set to be the characters in string, which I will convert to bytes"
	
	safeSet := string asByteArray ! !

!ZnPercentEncoder methodsFor: 'private' stamp: 'SvenVanCaekenberghe 10/23/2013 18:56'!
writeHex: integer to: stream
	integer printOn: stream base: 16 length: 2 padded: true! !

ZnSimplifiedByteEncoder initialize!

ZnBase64Encoder initialize!

ZnUTF8Encoder initialize!
